class MarkdownTree:

    def __init__(self, raw_txt):
        self.raw = raw_txt
        split = self.raw.split("---\n", 2)
        body = split[2]
        self.front_matter = FrontMatter(split[1])
        self.sections = parse_body(body)
    
    def sections_by_level(self, level):
        """Return all sections at a certain level in a list."""
        section_level = []
        for s in self.sections:
            if s.level == level:
                section_level.append(s)
        return section_level

    def sections_by_title(self, txt):
        """Return sections with txt in the title."""
        sections = []
        for s in self.sections:
            if txt in s.title:
                sections.append(s)
        return sections
        
class FrontMatter:

    def __init__(self, txt):
        self.raw = txt
        self.variables = {}

        txt = txt.split(":")
        splits = [txt[0]]
        for i in range(1, len(txt)):
            splits = splits.copy() + txt[i].rsplit("\n", 1)
        
        for i in range(0, len(splits) - 2, 2):
            key = splits[i]
            value = splits[i+1].lstrip("\n -")
            self.variables[key] = value

class Section:

    def __init__(self, title, paragraphs, level=0):
        self.title = title
        self.paragraphs = paragraphs
        self.level = level

class Paragraph:

    def __init__(self, txt):
        self.txt = txt
        self._table = None
    
    def __str__(self):
        return self.txt

    @property
    def is_table(self):
        # A single line can't be a table
        if "\n" not in self.txt:
            return False
        #If each line of a paragraph starts with '|', it is a table.
        for line in self.txt.splitlines():
            if not line.startswith("|"):
                return False
        return True

    @property
    def table(self):
        if not self._table is None:
            return self._table
        if not self.is_table:
            return None
        self._table = Table(self.txt)
        return self._table

class Table:

    def __init__(self, txt):
        self.txt = txt
        rows = txt.splitlines()
        # Ignore the second line, which only includes dashes
        rows = [rows[0]] + rows[2:]
        rows = [row.split("|")[1:-1] for row in rows]
        rows = [[cell.strip() for cell in row] for row in rows]
        self.rows = rows
    
    def __str__(self):
        return self.txt

def parse_body(txt, level=0):
    """Return a list of sections"""

    txt = "\n" + txt.strip("\n ")
    if "\n#" in txt:
        parsed_body = []
        # I want a single newline at the beginning of the text
        split = txt.split("\n#")
        # If the text starts without a section title, put it in a section at level 0
        if split[0].strip("\n") != "" and level==0:
            title = ""
            paragraphs = split[0].split("\n\n")
            paragraphs = [p.strip("\n ") for p in paragraphs]
            paragraphs = [Paragraph(p) for p in paragraphs]
            parsed_body.append(Section(title, paragraphs, 0))
            split.pop(0)
        for t in split:
            parsed_body += parse_body(t, level+1)
        return parsed_body
    
    # I don't care about empty sections
    if txt.strip("\n ") == "":
        return []

    split = txt.strip("\n ").split("\n", 1)
    title = split[0]
    paragraphs = split[1].split("\n\n")
    paragraphs = [p.strip("\n ") for p in paragraphs]
    paragraphs = [Paragraph(p) for p in paragraphs]
    return [Section(title, paragraphs, level)]



import pythoncom

def com_arguments(arguments: list) -> list:
    """Ensures that optional arguments are correctly represented for COM calls.
    
    Optional arguments should be `None` until there are defined arguments.
    After the last define argument, all subsequent optional arguments should be
    represented as `pythoncom.Missing`.
    """
    com_arguments = []
    replacement_value = pythoncom.Missing
    for p in reversed(arguments):
        if p is None:
            com_arguments.insert(0, replacement_value)
        else:
            com_arguments.insert(0, p)
            replacement_value = None
    return com_arguments

def unwrap(obj):
    """Return the underlying COM object if available"""
    
    if hasattr(obj, "com_object"):
        return obj.com_object
    return obj



import random

from matita.office import powerpoint as pp

def squares():
    pp_app = pp.Application()
    pp_app.visible = True
    prs = pp_app.presentations.add()
    sld = prs.slides.Add(1, pp.ppLayoutBlank)

    for _ in range(1000):
        side = random.random() * prs.page_setup.slideheight / 3
        left = -side + random.random() * (side + prs.page_setup.slide_width)
        top = -side + random.random() * (side + prs.page_setup.slide_height)
        shp = sld.shapes.add_shape(pp.msoShapeRectangle, left, top, side, side)
        shp.line.visible = False
        shp.fill.fore_color.rgb = random.randint(0, 256 ** 3)
        eff = sld.timeline.main_sequence.add_effect(
            Shape=shp,
            effectId=pp.msoAnimEffectFly,
            Level=pp.msoAnimateLevelNone,
            trigger=pp.msoAnimTriggerAfterPrevious,
        )
        direction = random.choice([
            pp.msoAnimDirectionLeft,
            pp.msoAnimDirectionTop,
            pp.msoAnimDirectionRight,
            pp.msoAnimDirectionBottom
        ])
        eff.effect_parameters.direction = direction
        eff.timing.duration = 0.2

if __name__ == "__main__":
    squares()



from matita.office import word as wd

def hello_world():
    wd_app = wd.Application()
    wd_app.visible = True
    doc = wd_app.documents.add()
    par = doc.content.paragraphs.add()
    par.range.text = "Hello world!"

if __name__ == "__main__":
    hello_world()



from . import com_arguments, unwrap
from .office import *

import win32com.client

# BackstageGroupStyle enumeration
BackstageGroupStyleError = 2
BackstageGroupStyleNormal = 0
BackstageGroupStyleWarning = 1

# CertificateDetail enumeration
certdetAvailable = 0
certdetExpirationDate = 3
certdetIssuer = 2
certdetSubject = 1
certdetThumbprint = 4

# CertificateVerificationResults enumeration
certverresError = 0
certverresExpired = 5
certverresInvalid = 4
certverresRevoked = 6
certverresUntrusted = 7
certverresUnverified = 2
certverresValid = 3
certverresVerifying = 1

# ContentVerificationResults enumeration
contverresError = 0
contverresModified = 4
contverresUnverified = 2
contverresValid = 3
contverresVerifying = 1

# EncryptionCipherMode enumeration
cipherModeECB = 0
cipherModeCBC = 1

# EncryptionProviderDetail enumeration
encprovdetURL = 0
encprovdetAlgorithm = 1
encprovdetBlockCipher = 2
encprovdetCipherBlockSize = 3
encprovdetCipherMode = 4

# MailFormat enumeration
mfHTML = 2
mfPlainText = 1
mfRTF = 3

# MsoAlertCancelType enumeration
msoAlertCancelDefault = -1
msoAlertCancelFifth = 4
msoAlertCancelFirst = 0
msoAlertCancelFourth = 3
msoAlertCancelSecond = 1
msoAlertCancelThird = 2

# MsoAlertIconType enumeration
msoAlertIconCritical = 1
msoAlertIconInfo = 4
msoAlertIconNoIcon = 0
msoAlertIconQuery = 2
msoAlertIconWarning = 3

# MsoAlignCmd enumeration
msoAlignBottoms = 5
msoAlignCenters = 1
msoAlignLefts = 0
msoAlignMiddles = 4
msoAlignRights = 2
msoAlignTops = 3

# MsoAppLanguageID enumeration
msoLanguageIDExeMode = 4
msoLanguageIDHelp = 3
msoLanguageIDInstall = 1
msoLanguageIDUI = 2
msoLanguageIDUIPrevious = 5

# MsoArrowheadLength enumeration
msoArrowheadLengthMedium = 2
msoArrowheadLengthMixed = -2
msoArrowheadLong = 3
msoArrowheadShort = 1

# MsoArrowheadStyle enumeration
msoArrowheadDiamond = 5
msoArrowheadNone = 1
msoArrowheadOpen = 3
msoArrowheadOval = 6
msoArrowheadStealth = 4
msoArrowheadStyleMixed = -2
msoArrowheadTriangle = 2

# MsoArrowheadWidth enumeration
msoArrowheadNarrow = 1
msoArrowheadWide = 3
msoArrowheadWidthMedium = 2
msoArrowheadWidthMixed = -2

# MsoAutomationSecurity enumeration
msoAutomationSecurityByUI = 2
msoAutomationSecurityForceDisable = 3
msoAutomationSecurityLow = 1

# MsoAutoShapeType enumeration
msoShape10pointStar = 149
msoShape12pointStar = 150
msoShape16pointStar = 94
msoShape24pointStar = 95
msoShape32pointStar = 96
msoShape4pointStar = 91
msoShape5pointStar = 92
msoShape6pointStar = 147
msoShape7pointStar = 148
msoShape8pointStar = 93
msoShapeActionButtonBackorPrevious = 129
msoShapeActionButtonBeginning = 131
msoShapeActionButtonCustom = 125
msoShapeActionButtonDocument = 134
msoShapeActionButtonEnd = 132
msoShapeActionButtonForwardorNext = 130
msoShapeActionButtonHelp = 127
msoShapeActionButtonHome = 126
msoShapeActionButtonInformation = 128
msoShapeActionButtonMovie = 136
msoShapeActionButtonReturn = 133
msoShapeActionButtonSound = 135
msoShapeArc = 25
msoShapeBalloon = 137
msoShapeBentArrow = 41
msoShapeBentUpArrow = 44
msoShapeBevel = 15
msoShapeBlockArc = 20
msoShapeCan = 13
msoShapeChartPlus = 182
msoShapeChartStar = 181
msoShapeChartX = 180
msoShapeChevron = 52
msoShapeChord = 161
msoShapeCircularArrow = 60
msoShapeCloud = 179
msoShapeCloudCallout = 108
msoShapeCorner = 162
msoShapeCornerTabs = 169
msoShapeCross = 11
msoShapeCube = 14
msoShapeCurvedDownArrow = 48
msoShapeCurvedDownRibbon = 100
msoShapeCurvedLeftArrow = 46
msoShapeCurvedRightArrow = 45
msoShapeCurvedUpArrow = 47
msoShapeCurvedUpRibbon = 99
msoShapeDecagon = 144
msoShapeDiagonalStripe = 141
msoShapeDiamond = 4
msoShapeDodecagon = 146
msoShapeDonut = 18
msoShapeDoubleBrace = 27
msoShapeDoubleBracket = 26
msoShapeDoubleWave = 104
msoShapeDownArrow = 36
msoShapeDownArrowCallout = 56
msoShapeDownRibbon = 98
msoShapeExplosion1 = 89
msoShapeExplosion2 = 90
msoShapeFlowchartAlternateProcess = 62
msoShapeFlowchartCard = 75
msoShapeFlowchartCollate = 79
msoShapeFlowchartConnector = 73
msoShapeFlowchartData = 64
msoShapeFlowchartDecision = 63
msoShapeFlowchartDelay = 84
msoShapeFlowchartDirectAccessStorage = 87
msoShapeFlowchartDisplay = 88
msoShapeFlowchartDocument = 67
msoShapeFlowchartExtract = 81
msoShapeFlowchartInternalStorage = 66
msoShapeFlowchartMagneticDisk = 86
msoShapeFlowchartManualInput = 71
msoShapeFlowchartManualOperation = 72
msoShapeFlowchartMerge = 82
msoShapeFlowchartMultidocument = 68
msoShapeFlowchartOfflineStorage = 139
msoShapeFlowchartOffpageConnector = 74
msoShapeFlowchartOr = 78
msoShapeFlowchartPredefinedProcess = 65
msoShapeFlowchartPreparation = 70
msoShapeFlowchartProcess = 61
msoShapeFlowchartPunchedTape = 76
msoShapeFlowchartSequentialAccessStorage = 85
msoShapeFlowchartSort = 80
msoShapeFlowchartStoredData = 83
msoShapeFlowchartSummingJunction = 77
msoShapeFlowchartTerminator = 69
msoShapeFoldedCorner = 16
msoShapeFrame = 158
msoShapeFunnel = 174
msoShapeGear6 = 172
msoShapeGear9 = 173
msoShapeHalfFrame = 159
msoShapeHeart = 21
msoShapeHeptagon = 145
msoShapeHexagon = 10
msoShapeHorizontalScroll = 102
msoShapeIsoscelesTriangle = 7
msoShapeLeftArrow = 34
msoShapeLeftArrowCallout = 54
msoShapeLeftBrace = 31
msoShapeLeftBracket = 29
msoShapeLeftCircularArrow = 176
msoShapeLeftRightArrow = 37
msoShapeLeftRightArrowCallout = 57
msoShapeLeftRightCircularArrow = 177
msoShapeLeftRightRibbon = 140
msoShapeLeftRightUpArrow = 40
msoShapeLeftUpArrow = 43
msoShapeLightningBolt = 22
msoShapeLineCallout1 = 109
msoShapeLineCallout1AccentBar = 113
msoShapeLineCallout1BorderandAccentBar = 121
msoShapeLineCallout1NoBorder = 117
msoShapeLineCallout2 = 110
msoShapeLineCallout2AccentBar = 114
msoShapeLineCallout2BorderandAccentBar = 122
msoShapeLineCallout2NoBorder = 118
msoShapeLineCallout3 = 111
msoShapeLineCallout3AccentBar = 115
msoShapeLineCallout3BorderandAccentBar = 123
msoShapeLineCallout3NoBorder = 119
msoShapeLineCallout4 = 112
msoShapeLineCallout4AccentBar = 116
msoShapeLineCallout4BorderandAccentBar = 124
msoShapeLineCallout4NoBorder = 120
msoShapeLineInverse = 183
msoShapeMathDivide = 166
msoShapeMathEqual = 167
msoShapeMathMinus = 164
msoShapeMathMultiply = 165
msoShapeMathNotEqual = 168
msoShapeMathPlus = 163
msoShapeMixed = -2
msoShapeMoon = 24
msoShapeNonIsoscelesTrapezoid = 143
msoShapeNoSymbol = 19
msoShapeNotchedRightArrow = 50
msoShapeNotPrimitive = 138
msoShapeOctagon = 6
msoShapeOval = 9
msoShapeOvalCallout = 107
msoShapeParallelogram = 2
msoShapePentagon = 51
msoShapePie = 142
msoShapePieWedge = 175
msoShapePlaque = 28
msoShapePlaqueTabs = 171
msoShapeQuadArrow = 39
msoShapeQuadArrowCallout = 59
msoShapeRectangle = 1
msoShapeRectangularCallout = 105
msoShapeRegularPentagon = 12
msoShapeRightArrow = 33
msoShapeRightArrowCallout = 53
msoShapeRightBrace = 32
msoShapeRightBracket = 30
msoShapeRightTriangle = 8
msoShapeRound1Rectangle = 151
msoShapeRound2DiagRectangle = 157
msoShapeRound2SameRectangle = 152
msoShapeRoundedRectangle = 5
msoShapeRoundedRectangularCallout = 106
msoShapeSmileyFace = 17
msoShapeSnip1Rectangle = 155
msoShapeSnip2DiagRectangle = 157
msoShapeSnip2SameRectangle = 156
msoShapeSnipRoundRectangle = 154
msoShapeSquareTabs = 170
msoShapeStripedRightArrow = 49
msoShapeSun = 23
msoShapeSwooshArrow = 178
msoShapeTear = 160
msoShapeTrapezoid = 3
msoShapeUpArrow = 35
msoShapeUpArrowCallout = 55
msoShapeUpDownArrow = 38
msoShapeUpDownArrowCallout = 58
msoShapeUpRibbon = 97
msoShapeUTurnArrow = 42
msoShapeVerticalScroll = 101
msoShapeWave = 103

# MsoAutoSize enumeration
msoAutoSizeMixed = -2
msoAutoSizeNone = 0
msoAutoSizeShapeToFitText = 1
msoAutoSizeTextToFitShape = 2

# MsoBackgroundStyleIndex enumeration
msoBackgroundStyle1 = 1
msoBackgroundStyle10 = 10
msoBackgroundStyle11 = 11
msoBackgroundStyle12 = 12
msoBackgroundStyle2 = 2
msoBackgroundStyle3 = 3
msoBackgroundStyle4 = 4
msoBackgroundStyle5 = 5
msoBackgroundStyle6 = 6
msoBackgroundStyle7 = 7
msoBackgroundStyle8 = 8
msoBackgroundStyle9 = 9
msoBackgroundStyleMixed = -2
msoBackgroundStyleNone = 0

# MsoBarPosition enumeration
msoBarBottom = 3
msoBarFloating = 4
msoBarLeft = 0
msoBarMenuBar = 6
msoBarPopup = 5
msoBarRight = 2
msoBarTop = 1

# MsoBarProtection enumeration
msoBarNoChangeDock = 16
msoBarNoChangeVisible = 8
msoBarNoCustomize = 1
msoBarNoHorizontalDock = 64
msoBarNoMove = 4
msoBarNoProtection = 0
msoBarNoResize = 2
msoBarNoVerticalDock = 32

# MsoBarRow enumeration
msoBarRowFirst = 0
msoBarRowLast = -1

# MsoBarType enumeration
msoBarTypeMenuBar = 1
msoBarTypeNormal = 0
msoBarTypePopup = 2

# MsoBaselineAlignment enumeration
msoBaselineAlignAuto = 5
msoBaselineAlignBaseline = 1
msoBaselineAlignCenter = 3
msoBaselineAlignFarEast50 = 4
msoBaselineAlignMixed = -2
msoBaselineAlignTop = 2

# MsoBevelType enumeration
msoBevelAngle = 6
msoBevelArtDeco = 13
msoBevelCircle = 3
msoBevelConvex = 8
msoBevelCoolSlant = 9
msoBevelCross = 5
msoBevelDivot = 10
msoBevelHardEdge = 12
msoBevelNone = 1
msoBevelRelaxedInset = 2
msoBevelRiblet = 11
msoBevelSlope = 4
msoBevelSoftRound = 7
msoBevelTypeMixed = -2

# MsoBlackWhiteMode enumeration
msoBlackWhiteAutomatic = 1
msoBlackWhiteBlack = 8
msoBlackWhiteBlackTextAndLine = 6
msoBlackWhiteDontShow = 10
msoBlackWhiteGrayOutline = 5
msoBlackWhiteGrayScale = 2
msoBlackWhiteHighContrast = 7
msoBlackWhiteInverseGrayScale = 4
msoBlackWhiteLightGrayScale = 3
msoBlackWhiteMixed = -2
msoBlackWhiteWhite = 9

# MsoBlogCategorySupport enumeration
msoBlogMultipleCategories = 2
msoBlogNoCategories = 0
msoBlogOneCategory = 1

# MsoBlogImageType enumeration
msoBlogImageTypeGIF = 2
msoBlogImageTypeJPEG = 1
msoBlogImageTypePNG = 3

# MsoBulletType enumeration
msoBulletMixed = -2
msoBulletNone = 0
msoBulletNumbered = 2
msoBulletPicture = 3
msoBulletUnnumbered = 1

# MsoButtonState enumeration
msoButtonDown = -1
msoButtonMixed = 2
msoButtonUp = 0

# MsoButtonStyle enumeration
msoButtonAutomatic = 0
msoButtonCaption = 2
msoButtonIcon = 1
msoButtonIconAndCaption = 3
msoButtonIconAndCaptionBelow = 11
msoButtonIconAndWrapCaption = 7
msoButtonIconAndWrapCaptionBelow = 15
msoButtonWrapCaption = 14

# MsoCalloutAngleType enumeration
msoCalloutAngle30 = 2
msoCalloutAngle45 = 3
msoCalloutAngle60 = 4
msoCalloutAngle90 = 5
msoCalloutAngleAutomatic = 1
msoCalloutAngleMixed = -2

# MsoCalloutDropType enumeration
msoCalloutDropBottom = 4
msoCalloutDropCenter = 3
msoCalloutDropCustom = 1
msoCalloutDropMixed = -2
msoCalloutDropTop = 2

# MsoCalloutType enumeration
msoCalloutFour = 4
msoCalloutMixed = -2
msoCalloutOne = 1
msoCalloutThree = 3
msoCalloutTwo = 2

# MsoCharacterSet enumeration
msoCharacterSetArabic = 1
msoCharacterSetCyrillic = 2
msoCharacterSetEnglishWesternEuropeanOtherLatinScript = 3
msoCharacterSetGreek = 4
msoCharacterSetHebrew = 5
msoCharacterSetJapanese = 6
msoCharacterSetKorean = 7
msoCharacterSetMultilingualUnicode = 8
msoCharacterSetSimplifiedChinese = 9
msoCharacterSetThai = 10
msoCharacterSetTraditionalChinese = 11
msoCharacterSetVietnamese = 12

# MsoChartElementType enumeration
msoElementChartFloorNone = 1200
msoElementChartFloorShow = 1201
msoElementChartTitleAboveChart = 2
msoElementChartTitleCenteredOverlay = 1
msoElementChartTitleNone = 0
msoElementChartWallNone = 1100
msoElementChartWallShow = 1101
msoElementDataLabelBestFit = 210
msoElementDataLabelBottom = 209
msoElementDataLabelCallout = 211
msoElementDataLabelCenter = 202
msoElementDataLabelInsideBase = 204
msoElementDataLabelInsideEnd = 203
msoElementDataLabelLeft = 206
msoElementDataLabelNone = 200
msoElementDataLabelOutSideEnd = 205
msoElementDataLabelRight = 207
msoElementDataLabelShow = 201
msoElementDataLabelTop = 208
msoElementDataTableNone = 500
msoElementDataTableShow = 501
msoElementDataTableWithLegendKeys = 502
msoElementErrorBarNone = 700
msoElementErrorBarPercentage = 702
msoElementErrorBarStandardDeviation = 703
msoElementErrorBarStandardError = 701
msoElementLegendBottom = 104
msoElementLegendLeft = 103
msoElementLegendLeftOverlay = 106
msoElementLegendNone = 100
msoElementLegendRight = 101
msoElementLegendRightOverlay = 105
msoElementLegendTop = 102
msoElementLineDropHiLoLine = 804
msoElementLineDropLine = 801
msoElementLineHiLoLine = 802
msoElementLineNone = 800
msoElementLineSeriesLine = 803
msoElementPlotAreaNone = 1000
msoElementPlotAreaShow = 1001
msoElementPrimaryCategoryAxisBillions = 374
msoElementPrimaryCategoryAxisLogScale = 375
msoElementPrimaryCategoryAxisMillions = 373
msoElementPrimaryCategoryAxisNone = 348
msoElementPrimaryCategoryAxisReverse = 351
msoElementPrimaryCategoryAxisShow = 349
msoElementPrimaryCategoryAxisThousands = 372
msoElementPrimaryCategoryAxisTitleAdjacentToAxis = 301
msoElementPrimaryCategoryAxisTitleBelowAxis = 302
msoElementPrimaryCategoryAxisTitleHorizontal = 305
msoElementPrimaryCategoryAxisTitleNone = 300
msoElementPrimaryCategoryAxisTitleRotated = 303
msoElementPrimaryCategoryAxisTitleVertical = 304
msoElementPrimaryCategoryAxisWithoutLabels = 350
msoElementPrimaryCategoryGridLinesMajor = 334
msoElementPrimaryCategoryGridLinesMinor = 333
msoElementPrimaryCategoryGridLinesMinorMajor = 335
msoElementPrimaryCategoryGridLinesNone = 332
msoElementPrimaryValueAxisBillions = 356
msoElementPrimaryValueAxisLogScale = 357
msoElementPrimaryValueAxisMillions = 355
msoElementPrimaryValueAxisNone = 352
msoElementPrimaryValueAxisShow = 353
msoElementPrimaryValueAxisThousands = 354
msoElementPrimaryValueAxisTitleAdjacentToAxis = 307
msoElementPrimaryValueAxisTitleBelowAxis = 308
msoElementPrimaryValueAxisTitleHorizontal = 311
msoElementPrimaryValueAxisTitleNone = 306
msoElementPrimaryValueAxisTitleRotated = 309
msoElementPrimaryValueAxisTitleVertical = 310
msoElementPrimaryValueGridLinesMajor = 330
msoElementPrimaryValueGridLinesMinor = 329
msoElementPrimaryValueGridLinesMinorMajor = 331
msoElementPrimaryValueGridLinesNone = 328
msoElementSecondaryCategoryAxisBillions = 378
msoElementSecondaryCategoryAxisLogScale = 379
msoElementSecondaryCategoryAxisMillions = 377
msoElementSecondaryCategoryAxisNone = 358
msoElementSecondaryCategoryAxisReverse = 361
msoElementSecondaryCategoryAxisShow = 359
msoElementSecondaryCategoryAxisThousands = 376
msoElementSecondaryCategoryAxisTitleAdjacentToAxis = 313
msoElementSecondaryCategoryAxisTitleBelowAxis = 314
msoElementSecondaryCategoryAxisTitleHorizontal = 317
msoElementSecondaryCategoryAxisTitleNone = 312
msoElementSecondaryCategoryAxisTitleRotated = 315
msoElementSecondaryCategoryAxisTitleVertical = 316
msoElementSecondaryCategoryAxisWithoutLabels = 360
msoElementSecondaryCategoryGridLinesMajor = 342
msoElementSecondaryCategoryGridLinesMinor = 341
msoElementSecondaryCategoryGridLinesMinorMajor = 343
msoElementSecondaryCategoryGridLinesNone = 340
msoElementSecondaryValueAxisBillions = 366
msoElementSecondaryValueAxisLogScale = 367
msoElementSecondaryValueAxisMillions = 365
msoElementSecondaryValueAxisNone = 362
msoElementSecondaryValueAxisShow = 363
msoElementSecondaryValueAxisThousands = 364
msoElementSecondaryValueAxisTitleAdjacentToAxis = 319
msoElementSecondaryValueAxisTitleBelowAxis = 320
msoElementSecondaryValueAxisTitleHorizontal = 323
msoElementSecondaryValueAxisTitleNone = 318
msoElementSecondaryValueAxisTitleRotated = 321
msoElementSecondaryValueAxisTitleVertical = 322
msoElementSecondaryValueGridLinesMajor = 338
msoElementSecondaryValueGridLinesMinor = 337
msoElementSecondaryValueGridLinesMinorMajor = 339
msoElementSecondaryValueGridLinesNone = 336
msoElementSeriesAxisGridLinesMajor = 346
msoElementSeriesAxisGridLinesMinor = 345
msoElementSeriesAxisGridLinesMinorMajor = 347
msoElementSeriesAxisGridLinesNone = 344
msoElementSeriesAxisNone = 368
msoElementSeriesAxisReverse = 371
msoElementSeriesAxisShow = 369
msoElementSeriesAxisTitleHorizontal = 327
msoElementSeriesAxisTitleNone = 324
msoElementSeriesAxisTitleRotated = 325
msoElementSeriesAxisTitleVertical = 326
msoElementSeriesAxisWithoutLabeling = 370
msoElementTrendlineAddExponential = 602
msoElementTrendlineAddLinear = 601
msoElementTrendlineAddLinearForecast = 603
msoElementTrendlineAddTwoPeriodMovingAverage = 604
msoElementTrendlineNone = 600
msoElementUpDownBarsNone = 900
msoElementUpDownBarsShow = 901

# MsoClipboardFormat enumeration
msoClipboardFormatHTML = 2
msoClipboardFormatMixed = -2
msoClipboardFormatNative = 1
msoClipboardFormatPlainText = 4
msoClipboardFormatRTF = 3

# MsoColorType enumeration
msoColorTypeCMS = 4
msoColorTypeCMYK = 3
msoColorTypeInk = 5
msoColorTypeMixed = -2
msoColorTypeRGB = 1
msoColorTypeScheme = 2

# MsoComboStyle enumeration
msoComboLabel = 1
msoComboNormal = 0

# MsoCommandBarButtonHyperlinkType enumeration
msoCommandBarButtonHyperlinkInsertPicture = 2
msoCommandBarButtonHyperlinkNone = 0
msoCommandBarButtonHyperlinkOpen = 1

# MsoConnectorType enumeration
msoConnectorCurve = 3
msoConnectorElbow = 2
msoConnectorStraight = 1
msoConnectorTypeMixed = -2

# MsoContactCardAddressType enumeration
msoContactCardAddressTypeUnknown = 0
msoContactCardAddressTypeOutlook = 1
msoContactCardAddressTypeSMTP = 2
msoContactCardAddressTypeIM = 1

# MsoContactCardStyle enumeration
msoContactCardFull = 1
msoContactCardHover = 0

# MsoContactCardType enumeration
msoContactCardTypeEnterpriseContact = 0
msoContactCardTypePersonalContact = 1
msoContactCardTypeUnknownContact = 2
msoContactCardTypeEnterpriseGroup = 3
msoContactCardTypePersonalDistributionList = 4

# MsoControlOLEUsage enumeration
msoControlOLEUsageBoth = 3
msoControlOLEUsageClient = 2
msoControlOLEUsageNeither = 0
msoControlOLEUsageServer = 1

# MsoControlType enumeration
msoControlActiveX = 22
msoControlAutoCompleteCombo = 26
msoControlButton = 1
msoControlButtonDropdown = 5
msoControlButtonPopup = 12
msoControlComboBox = 4
msoControlCustom = 0
msoControlDropdown = 3
msoControlEdit = 2
msoControlExpandingGrid = 16
msoControlGauge = 19
msoControlGenericDropdown = 8
msoControlGraphicCombo = 20
msoControlGraphicDropdown = 9
msoControlGraphicPopup = 11
msoControlGrid = 18
msoControlLabel = 15
msoControlLabelEx = 24
msoControlOCXDropdown = 7
msoControlPane = 21
msoControlPopup = 10
msoControlSpinner = 23
msoControlSplitButtonMRUPopup = 14
msoControlSplitButtonPopup = 13
msoControlSplitDropdown = 6
msoControlSplitExpandingGrid = 17
msoControlWorkPane = 25

# MsoCTPDockPosition enumeration
msoCTPDockPositionBottom = 3
msoCTPDockPositionFloating = 4
msoCTPDockPositionLeft = 0
msoCTPDockPositionRight = 2
msoCTPDockPositionTop = 1

# MsoCTPDockPositionRestrict enumeration
msoCTPDockPositionRestrictNoChange = 1
msoCTPDockPositionRestrictNoHorizontal = 2
msoCTPDockPositionRestrictNone = 0
msoCTPDockPositionRestrictNoVertical = 3

# MsoCustomXMLNodeType enumeration
msoCustomXMLNodeAttribute = 2
msoCustomXMLNodeCData = 4
msoCustomXMLNodeComment = 8
msoCustomXMLNodeDocument = 9
msoCustomXMLNodeElement = 1
msoCustomXMLNodeProcessingInstruction = 7
msoCustomXMLNodeText = 3

# MsoCustomXMLValidationErrorType enumeration
msoCustomXMLValidationErrorAutomaticallyCleared = 1
msoCustomXMLValidationErrorManual = 2
msoCustomXMLValidationErrorSchemaGenerated = 0

# MsoDateTimeFormat enumeration
msoDateTimeddddMMMMddyyyy = 2
msoDateTimedMMMMyyyy = 3
msoDateTimedMMMyy = 5
msoDateTimeFigureOut = 14
msoDateTimeFormatMixed = -2
msoDateTimeHmm = 10
msoDateTimehmmAMPM = 12
msoDateTimeHmmss = 11
msoDateTimehmmssAMPM = 13
msoDateTimeMdyy = 1
msoDateTimeMMddyyHmm = 8
msoDateTimeMMddyyhmmAMPM = 9
msoDateTimeMMMMdyyyy = 4
msoDateTimeMMMMyy = 6
msoDateTimeMMyy = 7

# MsoDistributeCmd enumeration
msoDistributeHorizontally = 0
msoDistributeVertically = 1

# MsoDocInspectorStatus enumeration
msoDocInspectorStatusDocOk = 0
msoDocInspectorStatusError = 2
msoDocInspectorStatusIssueFound = 1

# MsoDocProperties enumeration
msoPropertyTypeBoolean = 2
msoPropertyTypeDate = 3
msoPropertyTypeFloat = 5
msoPropertyTypeNumber = 1
msoPropertyTypeString = 4

# MsoEditingType enumeration
msoEditingAuto = 0
msoEditingCorner = 1
msoEditingSmooth = 2
msoEditingSymmetric = 3

# MsoEncoding enumeration
msoEncodingArabic = 1256
msoEncodingArabicASMO = 708
msoEncodingArabicAutoDetect = 51256
msoEncodingArabicTransparentASMO = 720
msoEncodingAutoDetect = 50001
msoEncodingBaltic = 1257
msoEncodingCentralEuropean = 1250
msoEncodingCyrillic = 1251
msoEncodingCyrillicAutoDetect = 51251
msoEncodingEBCDICArabic = 20420
msoEncodingEBCDICDenmarkNorway = 20277
msoEncodingEBCDICFinlandSweden = 20278
msoEncodingEBCDICFrance = 20297
msoEncodingEBCDICGermany = 20273
msoEncodingEBCDICGreek = 20423
msoEncodingEBCDICGreekModern = 875
msoEncodingEBCDICHebrew = 20424
msoEncodingEBCDICIcelandic = 20871
msoEncodingEBCDICInternational = 500
msoEncodingEBCDICItaly = 20280
msoEncodingEBCDICJapaneseKatakanaExtended = 20290
msoEncodingEBCDICJapaneseKatakanaExtendedAndJapanese = 50930
msoEncodingEBCDICJapaneseLatinExtendedAndJapanese = 50939
msoEncodingEBCDICKoreanExtended = 20833
msoEncodingEBCDICKoreanExtendedAndKorean = 50933
msoEncodingEBCDICLatinAmericaSpain = 20284
msoEncodingEBCDICMultilingualROECELatin2 = 870
msoEncodingEBCDICRussian = 20880
msoEncodingEBCDICSerbianBulgarian = 21025
msoEncodingEBCDICSimplifiedChineseExtendedAndSimplifiedChinese = 50935
msoEncodingEBCDICThai = 20838
msoEncodingEBCDICTurkish = 20905
msoEncodingEBCDICTurkishLatin5 = 1026
msoEncodingEBCDICUnitedKingdom = 20285
msoEncodingEBCDICUSCanada = 37
msoEncodingEBCDICUSCanadaAndJapanese = 50931
msoEncodingEBCDICUSCanadaAndTraditionalChinese = 50937
msoEncodingEUCChineseSimplifiedChinese = 51936
msoEncodingEUCJapanese = 51932
msoEncodingEUCKorean = 51949
msoEncodingEUCTaiwaneseTraditionalChinese = 51950
msoEncodingEuropa3 = 29001
msoEncodingExtAlphaLowercase = 21027
msoEncodingGreek = 1253
msoEncodingGreekAutoDetect = 51253
msoEncodingHebrew = 1255
msoEncodingHZGBSimplifiedChinese = 52936
msoEncodingIA5German = 20106
msoEncodingIA5IRV = 20105
msoEncodingIA5Norwegian = 20108
msoEncodingIA5Swedish = 20107
msoEncodingISCIIAssamese = 57006
msoEncodingISCIIBengali = 57003
msoEncodingISCIIDevanagari = 57002
msoEncodingISCIIGujarati = 57010
msoEncodingISCIIKannada = 57008
msoEncodingISCIIMalayalam = 57009
msoEncodingISCIIOriya = 57007
msoEncodingISCIIPunjabi = 57011
msoEncodingISCIITamil = 57004
msoEncodingISCIITelugu = 57005
msoEncodingISO2022CNSimplifiedChinese = 50229
msoEncodingISO2022CNTraditionalChinese = 50227
msoEncodingISO2022JPJISX02011989 = 50222
msoEncodingISO2022JPJISX02021984 = 50221
msoEncodingISO2022JPNoHalfwidthKatakana = 50220
msoEncodingISO2022KR = 50225
msoEncodingISO6937NonSpacingAccent = 20269
msoEncodingISO885915Latin9 = 28605
msoEncodingISO88591Latin1 = 28591
msoEncodingISO88592CentralEurope = 28592
msoEncodingISO88593Latin3 = 28593
msoEncodingISO88594Baltic = 28594
msoEncodingISO88595Cyrillic = 28595
msoEncodingISO88596Arabic = 28596
msoEncodingISO88597Greek = 28597
msoEncodingISO88598Hebrew = 28598
msoEncodingISO88598HebrewLogical = 38598
msoEncodingISO88599Turkish = 28599
msoEncodingJapaneseAutoDetect = 50932
msoEncodingJapaneseShiftJIS = 932
msoEncodingKOI8R = 20866
msoEncodingKOI8U = 21866
msoEncodingKorean = 949
msoEncodingKoreanAutoDetect = 50949
msoEncodingKoreanJohab = 1361
msoEncodingMacArabic = 10004
msoEncodingMacCroatia = 10082
msoEncodingMacCyrillic = 10007
msoEncodingMacGreek1 = 10006
msoEncodingMacHebrew = 10005
msoEncodingMacIcelandic = 10079
msoEncodingMacJapanese = 10001
msoEncodingMacKorean = 10003
msoEncodingMacLatin2 = 10029
msoEncodingMacRoman = 10000
msoEncodingMacRomania = 10010
msoEncodingMacSimplifiedChineseGB2312 = 10008
msoEncodingMacTraditionalChineseBig5 = 10002
msoEncodingMacTurkish = 10081
msoEncodingMacUkraine = 10017
msoEncodingOEMArabic = 864
msoEncodingOEMBaltic = 775
msoEncodingOEMCanadianFrench = 863
msoEncodingOEMCyrillic = 855
msoEncodingOEMCyrillicII = 866
msoEncodingOEMGreek437G = 737
msoEncodingOEMHebrew = 862
msoEncodingOEMIcelandic = 861
msoEncodingOEMModernGreek = 869
msoEncodingOEMMultilingualLatinI = 850
msoEncodingOEMMultilingualLatinII = 852
msoEncodingOEMNordic = 865
msoEncodingOEMPortuguese = 860
msoEncodingOEMTurkish = 857
msoEncodingOEMUnitedStates = 437
msoEncodingSimplifiedChineseAutoDetect = 50936
msoEncodingSimplifiedChineseGB18030 = 54936
msoEncodingSimplifiedChineseGBK = 936
msoEncodingT61 = 20261
msoEncodingTaiwanCNS = 20000
msoEncodingTaiwanEten = 20002
msoEncodingTaiwanIBM5550 = 20003
msoEncodingTaiwanTCA = 20001
msoEncodingTaiwanTeleText = 20004
msoEncodingTaiwanWang = 20005
msoEncodingThai = 874
msoEncodingTraditionalChineseAutoDetect = 50950
msoEncodingTraditionalChineseBig5 = 950
msoEncodingTurkish = 1254
msoEncodingUnicodeBigEndian = 1201
msoEncodingUnicodeLittleEndian = 1200
msoEncodingUSASCII = 20127
msoEncodingUTF7 = 65000
msoEncodingUTF8 = 65001
msoEncodingVietnamese = 1258
msoEncodingWestern = 1252

# MsoExtraInfoMethod enumeration
msoMethodGet = 0
msoMethodPost = 1

# MsoExtrusionColorType enumeration
msoExtrusionColorAutomatic = 1
msoExtrusionColorCustom = 2
msoExtrusionColorTypeMixed = -2

# MsoFarEastLineBreakLanguageID enumeration
msoFarEastLineBreakLanguageJapanese = 1041
msoFarEastLineBreakLanguageKorean = 1042
msoFarEastLineBreakLanguageSimplifiedChinese = 2052
msoFarEastLineBreakLanguageTraditionalChinese = 1028

# MsoFeatureInstall enumeration
msoFeatureInstallNone = 0
msoFeatureInstallOnDemand = 1
msoFeatureInstallOnDemandWithUI = 2

# MsoFileDialogType enumeration
msoFileDialogFilePicker = 3
msoFileDialogFolderPicker = 4
msoFileDialogOpen = 1
msoFileDialogSaveAs = 2

# MsoFileDialogView enumeration
msoFileDialogViewDetails = 2
msoFileDialogViewLargeIcons = 6
msoFileDialogViewList = 1
msoFileDialogViewPreview = 4
msoFileDialogViewProperties = 3
msoFileDialogViewSmallIcons = 7
msoFileDialogViewThumbnail = 5
msoFileDialogViewTiles = 9
msoFileDialogViewWebView = 8

# MsoFileNewAction enumeration
msoCreateNewFile = 1
msoEditFile = 0
msoOpenFile = 2

# MsoFileNewSection enumeration
msoBottomSection = 4
msoNew = 1
msoNewfromExistingFile = 2
msoNewfromTemplate = 3
msoOpenDocument = 0

# MsoFileValidationMode enumeration
msoFileValidationDefault = 0
msoFileValidationSkip = 1

# MsoFillType enumeration
msoFillBackground = 5
msoFillGradient = 3
msoFillMixed = -2
msoFillPatterned = 2
msoFillPicture = 6
msoFillSolid = 1
msoFillTextured = 4

# MsoFilterComparison enumeration
msoFilterComparisonContains = 8
msoFilterComparisonEqual = 0
msoFilterComparisonGreaterThan = 3
msoFilterComparisonGreaterThanEqual = 5
msoFilterComparisonIsBlank = 6
msoFilterComparisonIsNotBlank = 7
msoFilterComparisonLessThan = 2
msoFilterComparisonLessThanEqual = 4
msoFilterComparisonNotContains = 9
msoFilterComparisonNotEqual = 1

# MsoFilterConjunction enumeration
msoFilterConjunctionAnd = 0
msoFilterConjunctionOr = 1

# MsoFlipCmd enumeration
msoFlipHorizontal = 0
msoFlipVertical = 1

# MsoFontLanguageIndex enumeration
msoThemeComplexScript = 2
msoThemeEastAsian = 3
msoThemeLatin = 1

# MsoGradientColorType enumeration
msoGradientColorMixed = -2
msoGradientMultiColor = 4
msoGradientOneColor = 1
msoGradientPresetColors = 3
msoGradientTwoColors = 2

# MsoGradientStyle enumeration
msoGradientDiagonalDown = 4
msoGradientDiagonalUp = 3
msoGradientFromCenter = 7
msoGradientFromCorner = 5
msoGradientFromTitle = 6
msoGradientHorizontal = 1
msoGradientMixed = -2
msoGradientVertical = 2

# MsoGraphicStyleIndex enumeration
msoGraphicStylePreset1 = 1
msoGraphicStylePreset10 = 10
msoGraphicStylePreset11 = 11
msoGraphicStylePreset12 = 12
msoGraphicStylePreset13 = 13
msoGraphicStylePreset14 = 14
msoGraphicStylePreset15 = 15
msoGraphicStylePreset16 = 16
msoGraphicStylePreset17 = 17
msoGraphicStylePreset18 = 18
msoGraphicStylePreset19 = 19
msoGraphicStylePreset2 = 2
msoGraphicStylePreset20 = 20
msoGraphicStylePreset21 = 21
msoGraphicStylePreset22 = 22
msoGraphicStylePreset23 = 23
msoGraphicStylePreset24 = 24
msoGraphicStylePreset25 = 25
msoGraphicStylePreset26 = 26
msoGraphicStylePreset27 = 27
msoGraphicStylePreset28 = 28
msoGraphicStylePreset3 = 3
msoGraphicStylePreset4 = 4
msoGraphicStylePreset5 = 5
msoGraphicStylePreset6 = 6
msoGraphicStylePreset7 = 7
msoGraphicStylePreset8 = 8
msoGraphicStylePreset9 = 9
msoGraphicStyleMixed = -2
msoGraphicStyleNotAPreset = 0

# MsoHorizontalAnchor enumeration
msoAnchorCenter = 2
msoAnchorNone = 1
msoHorizontalAnchorMixed = -2

# MsoHyperlinkType enumeration
msoHyperlinkInlineShape = 2
msoHyperlinkRange = 0
msoHyperlinkShape = 1

# MsoLanguageID enumeration
msoLanguageIDAfrikaans = 1078
msoLanguageIDAlbanian = 1052
msoLanguageIDAmharic = 1118
msoLanguageIDArabic = 1025
msoLanguageIDArabicAlgeria = 5121
msoLanguageIDArabicBahrain = 15361
msoLanguageIDArabicEgypt = 3073
msoLanguageIDArabicIraq = 2049
msoLanguageIDArabicJordan = 11265
msoLanguageIDArabicKuwait = 13313
msoLanguageIDArabicLebanon = 12289
msoLanguageIDArabicLibya = 4097
msoLanguageIDArabicMorocco = 6145
msoLanguageIDArabicOman = 8193
msoLanguageIDArabicQatar = 16385
msoLanguageIDArabicSyria = 10241
msoLanguageIDArabicTunisia = 7169
msoLanguageIDArabicUAE = 14337
msoLanguageIDArabicYemen = 9217
msoLanguageIDArmenian = 1067
msoLanguageIDAssamese = 1101
msoLanguageIDAzeriCyrillic = 2092
msoLanguageIDAzeriLatin = 1068
msoLanguageIDBasque = 1069
msoLanguageIDBelgianDutch = 2067
msoLanguageIDBelgianFrench = 2060
msoLanguageIDBengali = 1093
msoLanguageIDBosnian = 4122
msoLanguageIDBosnianBosniaHerzegovinaCyrillic = 8218
msoLanguageIDBosnianBosniaHerzegovinaLatin = 5146
msoLanguageIDBrazilianPortuguese = 1046
msoLanguageIDBulgarian = 1026
msoLanguageIDBurmese = 1109
msoLanguageIDByelorussian = 1059
msoLanguageIDCatalan = 1027
msoLanguageIDCherokee = 1116
msoLanguageIDChineseHongKongSAR = 3076
msoLanguageIDChineseMacaoSAR = 5124
msoLanguageIDChineseSingapore = 4100
msoLanguageIDCroatian = 1050
msoLanguageIDCzech = 1029
msoLanguageIDDanish = 1030
msoLanguageIDDivehi = 1125
msoLanguageIDDutch = 1043
msoLanguageIDEdo = 1126
msoLanguageIDEnglishAUS = 3081
msoLanguageIDEnglishBelize = 10249
msoLanguageIDEnglishCanadian = 4105
msoLanguageIDEnglishCaribbean = 9225
msoLanguageIDEnglishIndonesia = 14345
msoLanguageIDEnglishIreland = 6153
msoLanguageIDEnglishJamaica = 8201
msoLanguageIDEnglishNewZealand = 5129
msoLanguageIDEnglishPhilippines = 13321
msoLanguageIDEnglishSouthAfrica = 7177
msoLanguageIDEnglishTrinidadTobago = 11273
msoLanguageIDEnglishUK = 2057
msoLanguageIDEnglishUS = 1033
msoLanguageIDEnglishZimbabwe = 12297
msoLanguageIDEstonian = 1061
msoLanguageIDFaeroese = 1080
msoLanguageIDFarsi = 1065
msoLanguageIDFilipino = 1124
msoLanguageIDFinnish = 1035
msoLanguageIDFrench = 1036
msoLanguageIDFrenchCameroon = 11276
msoLanguageIDFrenchCanadian = 3084
msoLanguageIDFrenchCotedIvoire = 12300
msoLanguageIDFrenchHaiti = 15372
msoLanguageIDFrenchLuxembourg = 5132
msoLanguageIDFrenchMali = 13324
msoLanguageIDFrenchMonaco = 6156
msoLanguageIDFrenchMorocco = 14348
msoLanguageIDFrenchReunion = 8204
msoLanguageIDFrenchSenegal = 10252
msoLanguageIDFrenchWestIndies = 7180
msoLanguageIDFrenchCongoDRC = 9228
msoLanguageIDFrisianNetherlands = 1122
msoLanguageIDFulfulde = 1127
msoLanguageIDGaelicIreland = 2108
msoLanguageIDGaelicScotland = 1084
msoLanguageIDGalician = 1110
msoLanguageIDGeorgian = 1079
msoLanguageIDGerman = 1031
msoLanguageIDGermanAustria = 3079
msoLanguageIDGermanLiechtenstein = 5127
msoLanguageIDGermanLuxembourg = 4103
msoLanguageIDGreek = 1032
msoLanguageIDGuarani = 1140
msoLanguageIDGujarati = 1095
msoLanguageIDHausa = 1128
msoLanguageIDHawaiian = 1141
msoLanguageIDHebrew = 1037
msoLanguageIDHindi = 1081
msoLanguageIDHungarian = 1038
msoLanguageIDIbibio = 1129
msoLanguageIDIcelandic = 1039
msoLanguageIDIgbo = 1136
msoLanguageIDIndonesian = 1057
msoLanguageIDInuktitut = 1117
msoLanguageIDItalian = 1040
msoLanguageIDJapanese = 1041
msoLanguageIDKannada = 1099
msoLanguageIDKanuri = 1137
msoLanguageIDKashmiri = 1120
msoLanguageIDKashmiriDevanagari = 2144
msoLanguageIDKazakh = 1087
msoLanguageIDKhmer = 1107
msoLanguageIDKirghiz = 1088
msoLanguageIDKonkani = 1111
msoLanguageIDKorean = 1042
msoLanguageIDKyrgyz = 1088
msoLanguageIDLao = 1108
msoLanguageIDLatin = 1142
msoLanguageIDLatvian = 1062
msoLanguageIDLithuanian = 1063
msoLanguageIDMacedonianFYROM = 1071
msoLanguageIDMalayalam = 1100
msoLanguageIDMalayBruneiDarussalam = 2110
msoLanguageIDMalaysian = 1086
msoLanguageIDMaltese = 1082
msoLanguageIDManipuri = 1112
msoLanguageIDMaori = 1153
msoLanguageIDMarathi = 1102
msoLanguageIDMexicanSpanish = 2058
msoLanguageIDMixed = -2
msoLanguageIDMongolian = 1104
msoLanguageIDNepali = 1121
msoLanguageIDNone = 0
msoLanguageIDNoProofing = 1024
msoLanguageIDNorwegianBokmol = 1044
msoLanguageIDNorwegianNynorsk = 2068
msoLanguageIDOriya = 1096
msoLanguageIDOromo = 1138
msoLanguageIDPashto = 1123
msoLanguageIDPolish = 1045
msoLanguageIDPortuguese = 2070
msoLanguageIDPunjabi = 1094
msoLanguageIDQuechuaBolivia = 1131
msoLanguageIDQuechuaEcuador = 2155
msoLanguageIDQuechuaPeru = 3179
msoLanguageIDRhaetoRomanic = 1047
msoLanguageIDRomanian = 1048
msoLanguageIDRomanianMoldova = 2072
msoLanguageIDRussian = 1049
msoLanguageIDRussianMoldova = 2073
msoLanguageIDSamiLappish = 1083
msoLanguageIDSanskrit = 1103
msoLanguageIDSepedi = 1132
msoLanguageIDSerbianBosniaHerzegovinaCyrillic = 7194
msoLanguageIDSerbianBosniaHerzegovinaLatin = 6170
msoLanguageIDSerbianCyrillic = 3098
msoLanguageIDSerbianLatin = 2074
msoLanguageIDSesotho = 1072
msoLanguageIDSimplifiedChinese = 2052
msoLanguageIDSindhi = 1113
msoLanguageIDSindhiPakistan = 2137
msoLanguageIDSinhalese = 1115
msoLanguageIDSlovak = 1051
msoLanguageIDSlovenian = 1060
msoLanguageIDSomali = 1143
msoLanguageIDSorbian = 1070
msoLanguageIDSpanish = 1034
msoLanguageIDSpanishArgentina = 11274
msoLanguageIDSpanishBolivia = 16394
msoLanguageIDSpanishChile = 13322
msoLanguageIDSpanishColombia = 9226
msoLanguageIDSpanishCostaRica = 5130
msoLanguageIDSpanishDominicanRepublic = 7178
msoLanguageIDSpanishEcuador = 12298
msoLanguageIDSpanishElSalvador = 17418
msoLanguageIDSpanishGuatemala = 4106
msoLanguageIDSpanishHonduras = 18442
msoLanguageIDSpanishModernSort = 3082
msoLanguageIDSpanishNicaragua = 19466
msoLanguageIDSpanishPanama = 6154
msoLanguageIDSpanishParaguay = 15370
msoLanguageIDSpanishPeru = 10250
msoLanguageIDSpanishPuertoRico = 20490
msoLanguageIDSpanishUruguay = 14346
msoLanguageIDSpanishVenezuela = 8202
msoLanguageIDSutu = 1072
msoLanguageIDSwahili = 1089
msoLanguageIDSwedish = 1053
msoLanguageIDSwedishFinland = 2077
msoLanguageIDSwissFrench = 4108
msoLanguageIDSwissGerman = 2055
msoLanguageIDSwissItalian = 2064
msoLanguageIDSyriac = 1114
msoLanguageIDTajik = 1064
msoLanguageIDTamazight = 1119
msoLanguageIDTamazightLatin = 2143
msoLanguageIDTamil = 1097
msoLanguageIDTatar = 1092
msoLanguageIDTelugu = 1098
msoLanguageIDThai = 1054
msoLanguageIDTibetan = 1105
msoLanguageIDTigrignaEritrea = 2163
msoLanguageIDTigrignaEthiopic = 1139
msoLanguageIDTraditionalChinese = 1028
msoLanguageIDTsonga = 1073
msoLanguageIDTswana = 1074
msoLanguageIDTurkish = 1055
msoLanguageIDTurkmen = 1090
msoLanguageIDUkrainian = 1058
msoLanguageIDUrdu = 1056
msoLanguageIDUzbekCyrillic = 2115
msoLanguageIDUzbekLatin = 1091
msoLanguageIDVenda = 1075
msoLanguageIDVietnamese = 1066
msoLanguageIDWelsh = 1106
msoLanguageIDXhosa = 1076
msoLanguageIDYi = 1144
msoLanguageIDYiddish = 1085
msoLanguageIDYoruba = 1130
msoLanguageIDZulu = 1077

# MsoLightRigType enumeration
msoLightRigBalanced = 14
msoLightRigBrightRoom = 27
msoLightRigChilly = 22
msoLightRigContrasting = 18
msoLightRigFlat = 24
msoLightRigFlood = 17
msoLightRigFreezing = 23
msoLightRigGlow = 26
msoLightRigHarsh = 16
msoLightRigLegacyFlat1 = 1
msoLightRigLegacyFlat2 = 2
msoLightRigLegacyFlat3 = 3
msoLightRigLegacyFlat4 = 4
msoLightRigLegacyHarsh1 = 9
msoLightRigLegacyHarsh2 = 10
msoLightRigLegacyHarsh3 = 11
msoLightRigLegacyHarsh4 = 12
msoLightRigLegacyNormal1 = 5
msoLightRigLegacyNormal2 = 6
msoLightRigLegacyNormal3 = 7
msoLightRigLegacyNormal4 = 8
msoLightRigMixed = -2
msoLightRigMorning = 19
msoLightRigSoft = 15
msoLightRigSunrise = 20
msoLightRigSunset = 21
msoLightRigThreePoint = 13
msoLightRigTwoPoint = 25

# MsoLineDashStyle enumeration
msoLineDash = 4
msoLineDashDot = 5
msoLineDashDotDot = 6
msoLineDashStyleMixed = -2
msoLineLongDash = 7
msoLineLongDashDot = 8
msoLineRoundDot = 3
msoLineSolid = 1
msoLineSquareDot = 2

# MsoLineStyle enumeration
msoLineSingle = 1
msoLineStyleMixed = -2
msoLineThickBetweenThin = 5
msoLineThickThin = 4
msoLineThinThick = 3
msoLineThinThin = 2

# MsoMenuAnimation enumeration
msoMenuAnimationNone = 0
msoMenuAnimationRandom = 1
msoMenuAnimationSlide = 3
msoMenuAnimationUnfold = 2

# MsoMetaPropertyType enumeration
msoMetaPropertyTypeBoolean = 1
msoMetaPropertyTypeCalculated = 3
msoMetaPropertyTypeChoice = 2
msoMetaPropertyTypeComputed = 4
msoMetaPropertyTypeCurrency = 5
msoMetaPropertyTypeDateTime = 6
msoMetaPropertyTypeFillInChoice = 7
msoMetaPropertyTypeGuid = 8
msoMetaPropertyTypeInteger = 9
msoMetaPropertyTypeLookup = 10
msoMetaPropertyTypeMax = 19
msoMetaPropertyTypeMultiChoice = 12
msoMetaPropertyTypeMultiChoiceFillIn = 13
msoMetaPropertyTypeMultiChoiceLookup = 11
msoMetaPropertyTypeNote = 14
msoMetaPropertyTypeNumber = 15
msoMetaPropertyTypeText = 16
msoMetaPropertyTypeUnknown = 0
msoMetaPropertyTypeUrl = 17
msoMetaPropertyTypeUser = 18

# MsoMixedType enumeration
msoIntegerMixed = 32768
msoSingleMixed = -2147483648

# MsoMoveRow enumeration
msoMoveRowFirst = -4
msoMoveRowNbr = -1
msoMoveRowNext = -2
msoMoveRowPrev = -3

# MsoNumberedBulletStyle enumeration
msoBulletAlphaLCParenBoth = 8
msoBulletAlphaLCParenRight = 9
msoBulletAlphaLCPeriod = 0
msoBulletAlphaUCParenBoth = 10
msoBulletAlphaUCParenRight = 11
msoBulletAlphaUCPeriod = 1
msoBulletArabicAbjadDash = 24
msoBulletArabicAlphaDash = 23
msoBulletArabicDBPeriod = 29
msoBulletArabicDBPlain = 28
msoBulletArabicParenBoth = 12
msoBulletArabicParenRight = 2
msoBulletArabicPeriod = 3
msoBulletArabicPlain = 13
msoBulletCircleNumDBPlain = 18
msoBulletCircleNumWDBlackPlain = 20
msoBulletCircleNumWDWhitePlain = 19
msoBulletHebrewAlphaDash = 25
msoBulletHindiAlpha1Period = 40
msoBulletHindiAlphaPeriod = 36
msoBulletHindiNumParenRight = 39
msoBulletHindiNumPeriod = 37
msoBulletKanjiKoreanPeriod = 27
msoBulletKanjiKoreanPlain = 26
msoBulletKanjiSimpChinDBPeriod = 38
msoBulletRomanLCParenBoth = 4
msoBulletRomanLCParenRight = 5
msoBulletRomanLCPeriod = 6
msoBulletRomanUCParenBoth = 14
msoBulletRomanUCParenRight = 15
msoBulletRomanUCPeriod = 7
msoBulletSimpChinPeriod = 17
msoBulletSimpChinPlain = 16
msoBulletStyleMixed = -2
msoBulletThaiAlphaParenBoth = 32
msoBulletThaiAlphaParenRight = 31
msoBulletThaiAlphaPeriod = 30
msoBulletThaiNumParenBoth = 35
msoBulletThaiNumParenRight = 34
msoBulletThaiNumPeriod = 33
msoBulletTradChinPeriod = 22
msoBulletTradChinPlain = 21

# MsoOLEMenuGroup enumeration
msoOLEMenuGroupContainer = 2
msoOLEMenuGroupEdit = 1
msoOLEMenuGroupFile = 0
msoOLEMenuGroupHelp = 5
msoOLEMenuGroupNone = -1
msoOLEMenuGroupObject = 3
msoOLEMenuGroupWindow = 4

# MsoOrgChartLayoutType enumeration
msoOrgChartLayoutBothHanging = 2
msoOrgChartLayoutLeftHanging = 3
msoOrgChartLayoutMixed = -2
msoOrgChartLayoutRightHanging = 4
msoOrgChartLayoutStandard = 1

# MsoOrgChartOrientation enumeration
msoOrgChartOrientationMixed = -2
msoOrgChartOrientationVertical = 1

# MsoOrientation enumeration
msoOrientationHorizontal = 1
msoOrientationMixed = -2
msoOrientationVertical = 2

# MsoParagraphAlignment enumeration
msoAlignCenter = 1
msoAlignDistribute = 4
msoAlignJustify = 3
msoAlignJustifyLow = 6
msoAlignLeft = 0
msoAlignMixed = -2
msoAlignRight = 2
msoAlignThaiDistribute = 5

# MsoPathFormat enumeration
msoPathType1 = 1
msoPathType2 = 2
msoPathType3 = 3
msoPathType4 = 4
msoPathType5 = 5
msoPathType6 = 6
msoPathType7 = 7
msoPathType8 = 8
msoPathType9 = 9
msoPathTypeMixed = -2
msoPathTypeNone = 0

# MsoPatternType enumeration
msoPattern10Percent = 2
msoPattern20Percent = 3
msoPattern25Percent = 4
msoPattern30Percent = 5
msoPattern40Percent = 6
msoPattern50Percent = 7
msoPattern5Percent = 1
msoPattern60Percent = 8
msoPattern70Percent = 9
msoPattern75Percent = 10
msoPattern80Percent = 11
msoPattern90Percent = 12
msoPatternCross = 51
msoPatternDarkDownwardDiagonal = 15
msoPatternDarkHorizontal = 13
msoPatternDarkUpwardDiagonal = 16
msoPatternDarkVertical = 14
msoPatternDashedDownwardDiagonal = 28
msoPatternDashedHorizontal = 32
msoPatternDashedUpwardDiagonal = 27
msoPatternDashedVertical = 31
msoPatternDiagonalBrick = 40
msoPatternDiagonalCross = 54
msoPatternDivot = 46
msoPatternDottedDiamond = 24
msoPatternDottedGrid = 45
msoPatternDownwardDiagonal = 52
msoPatternHorizontal = 49
msoPatternHorizontalBrick = 35
msoPatternLargeCheckerBoard = 36
msoPatternLargeConfetti = 33
msoPatternLargeGrid = 34
msoPatternLightDownwardDiagonal = 21
msoPatternLightHorizontal = 19
msoPatternLightUpwardDiagonal = 22
msoPatternLightVertical = 20
msoPatternMixed = -2
msoPatternNarrowHorizontal = 30
msoPatternNarrowVertical = 29
msoPatternOutlinedDiamond = 41
msoPatternPlaid = 42
msoPatternShingle = 47
msoPatternSmallCheckerBoard = 17
msoPatternSmallConfetti = 37
msoPatternSmallGrid = 23
msoPatternSolidDiamond = 39
msoPatternSphere = 43
msoPatternTrellis = 18
msoPatternUpwardDiagonal = 53
msoPatternVertical = 50
msoPatternWave = 48
msoPatternWeave = 44
msoPatternWideDownwardDiagonal = 25
msoPatternWideUpwardDiagonal = 26
msoPatternZigZag = 38

# MsoPermission enumeration
msoPermissionChange = 15
msoPermissionEdit = 2
msoPermissionExtract = 8
msoPermissionFullControl = 64
msoPermissionObjModel = 32
msoPermissionPrint = 16
msoPermissionRead = 1
msoPermissionSave = 4
msoPermissionView = 1

# MsoPickerField enumeration
msoPickerFieldUnknown = 0
msoPickerFieldDateTime = 1
msoPickerFieldNumber = 2
msoPickerFieldText = 3
msoPickerFieldUser = 4
msoPickerFieldMax = 5

# MsoPictureColorType enumeration
msoPictureAutomatic = 1
msoPictureBlackAndWhite = 3
msoPictureGrayscale = 2
msoPictureMixed = -2
msoPictureWatermark = 4

# MsoPictureEffectType enumeration
msoEffectBackgroundRemoval = 1
msoEffectBlur = 2
msoEffectBrightnessContrast = 3
msoEffectCement = 4
msoEffectChalkSketch = 5
msoEffectColorTemperature = 6
msoEffectCrisscrossEtching = 7
msoEffectCutout = 8
msoEffectFilmGrain = 9
msoEffectGlass = 10
msoEffectGlowDiffused = 11
msoEffectGlowEdges = 12
msoEffectLightScreen = 13
msoEffectLineDrawing = 14
msoEffectMarker = 15
msoEffectMosaicBubbles = 16
msoEffectNone = 17
msoEffectPaintBrush = 18
msoEffectPaintStrokes = 19
msoEffectPastelsSmooth = 20
msoEffectPencilGrayscale = 21
msoEffectPencilSketch = 22
msoEffectPhotocopy = 23
msoEffectPlasticWrap = 24
msoEffectSaturation = 25
msoEffectSharpenSoften = 26
msoEffectTexturizer = 27
msoEffectWatercolorSponge = 28

# MsoPresetCamera enumeration
msoCameraIsometricBottomDown = 23
msoCameraIsometricBottomUp = 22
msoCameraIsometricLeftDown = 25
msoCameraIsometricLeftUp = 24
msoCameraIsometricOffAxis1Left = 28
msoCameraIsometricOffAxis1Right = 29
msoCameraIsometricOffAxis1Top = 30
msoCameraIsometricOffAxis2Left = 31
msoCameraIsometricOffAxis2Right = 32
msoCameraIsometricOffAxis2Top = 33
msoCameraIsometricOffAxis3Bottom = 36
msoCameraIsometricOffAxis3Left = 34
msoCameraIsometricOffAxis3Right = 35
msoCameraIsometricOffAxis4Bottom = 39
msoCameraIsometricOffAxis4Left = 37
msoCameraIsometricOffAxis4Right = 38
msoCameraIsometricRightDown = 27
msoCameraIsometricRightUp = 26
msoCameraIsometricTopDown = 21
msoCameraIsometricTopUp = 20
msoCameraLegacyObliqueBottom = 8
msoCameraLegacyObliqueBottomLeft = 7
msoCameraLegacyObliqueBottomRight = 9
msoCameraLegacyObliqueFront = 5
msoCameraLegacyObliqueLeft = 4
msoCameraLegacyObliqueRight = 6
msoCameraLegacyObliqueTop = 2
msoCameraLegacyObliqueTopLeft = 1
msoCameraLegacyObliqueTopRight = 3
msoCameraLegacyPerspectiveBottom = 17
msoCameraLegacyPerspectiveBottomLeft = 16
msoCameraLegacyPerspectiveBottomRight = 18
msoCameraLegacyPerspectiveFront = 14
msoCameraLegacyPerspectiveLeft = 13
msoCameraLegacyPerspectiveRight = 15
msoCameraLegacyPerspectiveTop = 11
msoCameraLegacyPerspectiveTopLeft = 10
msoCameraLegacyPerspectiveTopRight = 12
msoCameraObliqueBottom = 46
msoCameraObliqueBottomLeft = 45
msoCameraObliqueBottomRight = 47
msoCameraObliqueLeft = 43
msoCameraObliqueRight = 44
msoCameraObliqueTop = 41
msoCameraObliqueTopLeft = 40
msoCameraObliqueTopRight = 42
msoCameraOrthographicFront = 19
msoCameraPerspectiveAbove = 51
msoCameraPerspectiveAboveLeftFacing = 53
msoCameraPerspectiveAboveRightFacing = 54
msoCameraPerspectiveBelow = 52
msoCameraPerspectiveContrastingLeftFacing = 55
msoCameraPerspectiveContrastingRightFacing = 56
msoCameraPerspectiveFront = 48
msoCameraPerspectiveHeroicExtremeLeftFacing = 59
msoCameraPerspectiveHeroicExtremeRightFacing = 60
msoCameraPerspectiveHeroicLeftFacing = 57
msoCameraPerspectiveHeroicRightFacing = 58
msoCameraPerspectiveLeft = 49
msoCameraPerspectiveRelaxed = 61
msoCameraPerspectiveRelaxedModerately = 62
msoCameraPerspectiveRight = 50
msoPresetCameraMixed = -2

# MsoPresetExtrusionDirection enumeration
msoExtrusionBottom = 2
msoExtrusionBottomLeft = 3
msoExtrusionBottomRight = 1
msoExtrusionLeft = 6
msoExtrusionNone = 5
msoExtrusionRight = 4
msoExtrusionTop = 8
msoExtrusionTopLeft = 9
msoExtrusionTopRight = 7
msoPresetExtrusionDirectionMixed = -2

# MsoPresetGradientType enumeration
msoGradientBrass = 20
msoGradientCalmWater = 8
msoGradientChrome = 21
msoGradientChromeII = 22
msoGradientDaybreak = 4
msoGradientDesert = 6
msoGradientEarlySunset = 1
msoGradientFire = 9
msoGradientFog = 10
msoGradientGold = 18
msoGradientGoldII = 19
msoGradientHorizon = 5
msoGradientLateSunset = 2
msoGradientMahogany = 15
msoGradientMoss = 11
msoGradientNightfall = 3
msoGradientOcean = 7
msoGradientParchment = 14
msoGradientPeacock = 12
msoGradientRainbow = 16
msoGradientRainbowII = 17
msoGradientSapphire = 24
msoGradientSilver = 23
msoGradientWheat = 13
msoPresetGradientMixed = -2

# MsoPresetLightingDirection enumeration
msoLightingBottom = 8
msoLightingBottomLeft = 7
msoLightingBottomRight = 9
msoLightingLeft = 4
msoLightingNone = 5
msoLightingRight = 6
msoLightingTop = 2
msoLightingTopLeft = 1
msoLightingTopRight = 3
msoPresetLightingDirectionMixed = -2

# MsoPresetLightingSoftness enumeration
msoLightingBright = 3
msoLightingDim = 1
msoLightingNormal = 2
msoPresetLightingSoftnessMixed = -2

# MsoPresetMaterial enumeration
msoMaterialClear = 13
msoMaterialDarkEdge = 11
msoMaterialFlat = 14
msoMaterialMatte = 1
msoMaterialMatte2 = 5
msoMaterialMetal = 3
msoMaterialMetal2 = 7
msoMaterialPlastic = 2
msoMaterialPlastic2 = 6
msoMaterialPowder = 10
msoMaterialSoftEdge = 12
msoMaterialSoftMetal = 15
msoMaterialTranslucentPowder = 9
msoMaterialWarmMatte = 8
msoMaterialWireFrame = 4
msoPresetMaterialMixed = -2

# MsoPresetTextEffect enumeration
msoTextEffect1 = 0
msoTextEffect10 = 9
msoTextEffect11 = 10
msoTextEffect12 = 11
msoTextEffect13 = 12
msoTextEffect14 = 13
msoTextEffect15 = 14
msoTextEffect16 = 15
msoTextEffect17 = 16
msoTextEffect18 = 17
msoTextEffect19 = 18
msoTextEffect2 = 1
msoTextEffect20 = 19
msoTextEffect21 = 20
msoTextEffect22 = 21
msoTextEffect23 = 22
msoTextEffect24 = 23
msoTextEffect25 = 24
msoTextEffect26 = 25
msoTextEffect27 = 26
msoTextEffect28 = 27
msoTextEffect29 = 28
msoTextEffect3 = 2
msoTextEffect30 = 29
msoTextEffect31 = 30
msoTextEffect32 = 31
msoTextEffect33 = 32
msoTextEffect34 = 33
msoTextEffect35 = 34
msoTextEffect36 = 35
msoTextEffect37 = 36
msoTextEffect38 = 37
msoTextEffect39 = 38
msoTextEffect4 = 3
msoTextEffect40 = 39
msoTextEffect41 = 40
msoTextEffect42 = 41
msoTextEffect43 = 42
msoTextEffect44 = 43
msoTextEffect45 = 44
msoTextEffect46 = 45
msoTextEffect47 = 46
msoTextEffect48 = 47
msoTextEffect49 = 48
msoTextEffect5 = 4
msoTextEffect50 = 49
msoTextEffect6 = 5
msoTextEffect7 = 6
msoTextEffect8 = 7
msoTextEffect9 = 8
msoTextEffectMixed = -2

# MsoPresetTextEffectShape enumeration
msoTextEffectShapeArchDownCurve = 10
msoTextEffectShapeArchDownPour = 14
msoTextEffectShapeArchUpCurve = 9
msoTextEffectShapeArchUpPour = 13
msoTextEffectShapeButtonCurve = 12
msoTextEffectShapeButtonPour = 16
msoTextEffectShapeCanDown = 20
msoTextEffectShapeCanUp = 19
msoTextEffectShapeCascadeDown = 40
msoTextEffectShapeCascadeUp = 39
msoTextEffectShapeChevronDown = 6
msoTextEffectShapeChevronUp = 5
msoTextEffectShapeCircleCurve = 11
msoTextEffectShapeCirclePour = 15
msoTextEffectShapeCurveDown = 18
msoTextEffectShapeCurveUp = 17
msoTextEffectShapeDeflate = 26
msoTextEffectShapeDeflateBottom = 28
msoTextEffectShapeDeflateInflate = 31
msoTextEffectShapeDeflateInflateDeflate = 32
msoTextEffectShapeDeflateTop = 30
msoTextEffectShapeDoubleWave1 = 23
msoTextEffectShapeDoubleWave2 = 24
msoTextEffectShapeFadeDown = 36
msoTextEffectShapeFadeLeft = 34
msoTextEffectShapeFadeRight = 33
msoTextEffectShapeFadeUp = 35
msoTextEffectShapeInflate = 25
msoTextEffectShapeInflateBottom = 27
msoTextEffectShapeInflateTop = 29
msoTextEffectShapeMixed = -2
msoTextEffectShapePlainText = 1
msoTextEffectShapeRingInside = 7
msoTextEffectShapeRingOutside = 8
msoTextEffectShapeSlantDown = 38
msoTextEffectShapeSlantUp = 37
msoTextEffectShapeStop = 2
msoTextEffectShapeTriangleDown = 4
msoTextEffectShapeTriangleUp = 3
msoTextEffectShapeWave1 = 21
msoTextEffectShapeWave2 = 22

# MsoPresetTexture enumeration
msoPresetTextureMixed = -2
msoTextureBlueTissuePaper = 17
msoTextureBouquet = 20
msoTextureBrownMarble = 11
msoTextureCanvas = 2
msoTextureCork = 21
msoTextureDenim = 3
msoTextureFishFossil = 7
msoTextureGranite = 12
msoTextureGreenMarble = 9
msoTextureMediumWood = 24
msoTextureNewsprint = 13
msoTextureOak = 23
msoTexturePaperBag = 6
msoTexturePapyrus = 1
msoTextureParchment = 15
msoTexturePinkTissuePaper = 18
msoTexturePurpleMesh = 19
msoTextureRecycledPaper = 14
msoTextureSand = 8
msoTextureStationery = 16
msoTextureWalnut = 22
msoTextureWaterDroplets = 5
msoTextureWhiteMarble = 10
msoTextureWovenMat = 4

# MsoPresetThreeDFormat enumeration
msoPresetThreeDFormatMixed = -2
msoThreeD1 = 1
msoThreeD10 = 10
msoThreeD11 = 11
msoThreeD12 = 12
msoThreeD13 = 13
msoThreeD14 = 14
msoThreeD15 = 15
msoThreeD16 = 16
msoThreeD17 = 17
msoThreeD18 = 18
msoThreeD19 = 19
msoThreeD2 = 2
msoThreeD20 = 20
msoThreeD3 = 3
msoThreeD4 = 4
msoThreeD5 = 5
msoThreeD6 = 6
msoThreeD7 = 7
msoThreeD8 = 8
msoThreeD9 = 9

# MsoRecolorType enumeration
msoRecolorType1 = 1
msoRecolorType10 = 10
msoRecolorType2 = 2
msoRecolorType3 = 3
msoRecolorType4 = 4
msoRecolorType5 = 5
msoRecolorType6 = 6
msoRecolorType7 = 7
msoRecolorType8 = 8
msoRecolorType9 = 9
msoRecolorTypeMixed = -2
msoRecolorTypeNone = 0

# MsoReflectionType enumeration
msoReflectionType1 = 1
msoReflectionType2 = 2
msoReflectionType3 = 3
msoReflectionType4 = 4
msoReflectionType5 = 5
msoReflectionType6 = 6
msoReflectionType7 = 7
msoReflectionType8 = 8
msoReflectionType9 = 9
msoReflectionTypeMixed = -2
msoReflectionTypeNone = 0

# MsoRelativeNodePosition enumeration
msoAfterLastSibling = 4
msoAfterNode = 2
msoBeforeFirstSibling = 3
msoBeforeNode = 1

# MsoScaleFrom enumeration
msoScaleFromBottomRight = 2
msoScaleFromMiddle = 1
msoScaleFromTopLeft = 0

# MsoScreenSize enumeration
msoScreenSize1024x768 = 4
msoScreenSize1152x882 = 5
msoScreenSize1152x900 = 6
msoScreenSize1280x1024 = 7
msoScreenSize1600x1200 = 8
msoScreenSize1800x1440 = 9
msoScreenSize1920x1200 = 10
msoScreenSize544x376 = 0
msoScreenSize640x480 = 1
msoScreenSize720x512 = 2
msoScreenSize800x600 = 3

# MsoSegmentType enumeration
msoSegmentCurve = 1
msoSegmentLine = 0

# MsoShadowStyle enumeration
msoShadowStyleInnerShadow = 1
msoShadowStyleMixed = -2
msoShadowStyleOuterShadow = 2

# MsoShadowType enumeration
msoShadow1 = 1
msoShadow10 = 10
msoShadow11 = 11
msoShadow12 = 12
msoShadow13 = 13
msoShadow14 = 14
msoShadow15 = 15
msoShadow16 = 16
msoShadow17 = 17
msoShadow18 = 18
msoShadow19 = 19
msoShadow2 = 2
msoShadow20 = 20
msoShadow3 = 3
msoShadow4 = 4
msoShadow5 = 5
msoShadow6 = 6
msoShadow7 = 7
msoShadow8 = 8
msoShadow9 = 9
msoShadowMixed = -2

# MsoShapeStyleIndex enumeration
msoLineStylePreset1 = 10001
msoLineStylePreset10 = 10010
msoLineStylePreset11 = 10011
msoLineStylePreset12 = 10012
msoLineStylePreset13 = 10013
msoLineStylePreset14 = 10014
msoLineStylePreset15 = 10015
msoLineStylePreset16 = 10016
msoLineStylePreset17 = 10017
msoLineStylePreset18 = 10018
msoLineStylePreset19 = 10019
msoLineStylePreset2 = 10002
msoLineStylePreset20 = 10020
msoLineStylePreset3 = 10003
msoLineStylePreset4 = 10004
msoLineStylePreset5 = 10005
msoLineStylePreset6 = 10006
msoLineStylePreset7 = 10007
msoLineStylePreset8 = 10008
msoLineStylePreset9 = 10009
msoShapeStylePreset1 = 1
msoShapeStylePreset10 = 10
msoShapeStylePreset11 = 11
msoShapeStylePreset12 = 12
msoShapeStylePreset13 = 13
msoShapeStylePreset14 = 14
msoShapeStylePreset15 = 15
msoShapeStylePreset16 = 16
msoShapeStylePreset17 = 17
msoShapeStylePreset18 = 18
msoShapeStylePreset19 = 19
msoShapeStylePreset2 = 2
msoShapeStylePreset20 = 20
msoShapeStylePreset3 = 3
msoShapeStylePreset4 = 4
msoShapeStylePreset5 = 5
msoShapeStylePreset6 = 6
msoShapeStylePreset7 = 7
msoShapeStylePreset8 = 8
msoShapeStylePreset9 = 9
msoShapeStyleMixed = -2
msoShapeStyleNotAPreset = 0

# MsoShapeType enumeration
mso3DModel = 30
msoAutoShape = 1
msoCallout = 2
msoCanvas = 20
msoChart = 3
msoComment = 4
msoContentApp = 27
msoDiagram = 21
msoEmbeddedOLEObject = 7
msoFormControl = 8
msoFreeform = 5
msoGraphic = 28
msoGroup = 6
msoIgxGraphic = 24
msoInk = 22
msoInkComment = 23
msoLine = 9
msoLinked3DModel = 31
msoLinkedGraphic = 29
msoLinkedOLEObject = 10
msoLinkedPicture = 11
msoMedia = 16
msoOLEControlObject = 12
msoPicture = 13
msoPlaceholder = 14
msoScriptAnchor = 18
msoShapeTypeMixed = -2
msoSlicer = 25
msoTable = 19
msoTextBox = 17
msoTextEffect = 15
msoWebVideo = 26

# MsoSharedWorkspaceTaskPriority enumeration
msoSharedWorkspaceTaskPriorityHigh = 1
msoSharedWorkspaceTaskPriorityLow = 3
msoSharedWorkspaceTaskPriorityNormal = 2

# MsoSharedWorkspaceTaskStatus enumeration
msoSharedWorkspaceTaskStatusCompleted = 3
msoSharedWorkspaceTaskStatusDeferred = 4
msoSharedWorkspaceTaskStatusInProgress = 2
msoSharedWorkspaceTaskStatusNotStarted = 1
msoSharedWorkspaceTaskStatusWaiting = 5

# MsoSignatureSubset enumeration
msoSignatureSubsetAll = 5
msoSignatureSubsetSignatureLines = 2
msoSignatureSubsetSignatureLinesSigned = 3
msoSignatureSubsetSignatureLinesUnsigned = 4
msoSignatureSubsetSignaturesAllSigs = 0
msoSignatureSubsetSignaturesNonVisible = 1

# MsoSmartArtNodePosition enumeration
msoSmartArtNodeAbove = 4
msoSmartArtNodeAfter = 2
msoSmartArtNodeBefore = 3
msoSmartArtNodeBelow = 5
msoSmartArtNodeDefault = 1

# MsoSmartArtNodeType enumeration
msoSmartArtNodeTypeAssistant = 2
msoSmartArtNodeTypeDefault = 1

# MsoSoftEdgeType enumeration
msoSoftEdgeType1 = 1
msoSoftEdgeType2 = 2
msoSoftEdgeType3 = 3
msoSoftEdgeType4 = 4
msoSoftEdgeType5 = 5
msoSoftEdgeType6 = 6
msoSoftEdgeTypeNone = 0
SoftEdgeTypeMixed = -2

# MsoSyncConflictResolutionType enumeration
msoSyncConflictClientWins = 0
msoSyncConflictMerge = 2
msoSyncConflictServerWins = 1

# MsoSyncErrorType enumeration
msoSyncErrorCouldNotCompare = 13
msoSyncErrorCouldNotConnect = 2
msoSyncErrorCouldNotOpen = 11
msoSyncErrorCouldNotResolve = 14
msoSyncErrorCouldNotUpdate = 12
msoSyncErrorFileInUse = 6
msoSyncErrorFileNotFound = 4
msoSyncErrorFileTooLarge = 5
msoSyncErrorNone = 0
msoSyncErrorNoNetwork = 15
msoSyncErrorOutOfSpace = 3
msoSyncErrorUnauthorizedUser = 1
msoSyncErrorUnknown = 16
msoSyncErrorUnknownDownload = 10
msoSyncErrorUnknownUpload = 9
msoSyncErrorVirusDownload = 8
msoSyncErrorVirusUpload = 7

# MsoSyncEventType enumeration
msoSyncEventDownloadFailed = 2
msoSyncEventDownloadInitiated = 0
msoSyncEventDownloadNoChange = 6
msoSyncEventDownloadSucceeded = 1
msoSyncEventOffline = 7
msoSyncEventUploadFailed = 5
msoSyncEventUploadInitiated = 3
msoSyncEventUploadSucceeded = 4

# MsoSyncStatusType enumeration
msoSyncStatusConflict = 4
msoSyncStatusError = 6
msoSyncStatusLatest = 1
msoSyncStatusLocalChanges = 3
msoSyncStatusNewerAvailable = 2
msoSyncStatusNoSharedWorkspace = 0
msoSyncStatusNotRoaming = 0
msoSyncStatusSuspended = 5

# MsoSyncVersionType enumeration
msoSyncVersionLastViewed = 0
msoSyncVersionServer = 1

# MsoTabStopType enumeration
msoTabStopCenter = 2
msoTabStopDecimal = 4
msoTabStopLeft = 1
msoTabStopMixed = -2
msoTabStopRight = 3

# MsoTargetBrowser enumeration
msoTargetBrowserIE4 = 2
msoTargetBrowserIE5 = 3
msoTargetBrowserIE6 = 4
msoTargetBrowserV3 = 0
msoTargetBrowserV4 = 1

# MsoTextCaps enumeration
msoAllCaps = 2
msoCapsMixed = -2
msoNoCaps = 0
msoSmallCaps = 1

# MsoTextChangeCase enumeration
msoCaseLower = 2
msoCaseSentence = 1
msoCaseTitle = 4
msoCaseToggle = 5
msoCaseUpper = 3

# MsoTextCharWrap enumeration
msoCharWrapMixed = -2
msoCustomCharWrap = 3
msoNoCharWrap = 0
msoStandardCharWrap = 1
msoStrictCharWrap = 2

# MsoTextDirection enumeration
msoTextDirectionLeftToRight = 1
msoTextDirectionMixed = -2
msoTextDirectionRightToLeft = 2

# MsoTextEffectAlignment enumeration
msoTextEffectAlignmentCentered = 2
msoTextEffectAlignmentLeft = 1
msoTextEffectAlignmentLetterJustify = 4
msoTextEffectAlignmentMixed = -2
msoTextEffectAlignmentRight = 3
msoTextEffectAlignmentStretchJustify = 6
msoTextEffectAlignmentWordJustify = 5

# MsoTextFontAlign enumeration
msoFontAlignAuto = 0
msoFontAlignBaseline = 3
msoFontAlignBottom = 4
msoFontAlignCenter = 2
msoFontAlignMixed = -2
msoFontAlignTop = 1

# MsoTextOrientation enumeration
msoTextOrientationDownward = 3
msoTextOrientationHorizontal = 1
msoTextOrientationHorizontalRotatedFarEast = 6
msoTextOrientationMixed = -2
msoTextOrientationUpward = 2
msoTextOrientationVertical = 5
msoTextOrientationVerticalFarEast = 4

# MsoTextStrike enumeration
msoDoubleStrike = 2
msoNoStrike = 0
msoSingleStrike = 1
msoStrikeMixed = -2

# MsoTextTabAlign enumeration
msoTabAlignCenter = 1
msoTabAlignDecimal = 3
msoTabAlignLeft = 0
msoTabAlignMixed = -2
msoTabAlignRight = 2

# MsoTextUnderlineType enumeration
msoNoUnderline = 0
msoUnderlineDashHeavyLine = 8
msoUnderlineDashLine = 7
msoUnderlineDashLongHeavyLine = 10
msoUnderlineDashLongLine = 9
msoUnderlineDotDashHeavyLine = 12
msoUnderlineDotDashLine = 11
msoUnderlineDotDotDashHeavyLine = 14
msoUnderlineDotDotDashLine = 13
msoUnderlineDottedHeavyLine = 6
msoUnderlineDottedLine = 5
msoUnderlineDoubleLine = 3
msoUnderlineHeavyLine = 4
msoUnderlineMixed = -2
msoUnderlineSingleLine = 2
msoUnderlineWavyDoubleLine = 17
msoUnderlineWavyHeavyLine = 16
msoUnderlineWavyLine = 15
msoUnderlineWords = 1

# MsoTextureAlignment enumeration
msoTextureAlignmentMixed = -2
msoTextureBottom = 7
msoTextureBottomLeft = 6
msoTextureBottomRight = 8
msoTextureCenter = 4
msoTextureLeft = 3
msoTextureRight = 5
msoTextureTop = 1
msoTextureTopLeft = 0
msoTextureTopRight = 2

# MsoTextureType enumeration
msoTexturePreset = 1
msoTextureTypeMixed = -2
msoTextureUserDefined = 2

# MsoThemeColorIndex enumeration
msoNotThemeColor = 0
msoThemeColorAccent1 = 5
msoThemeColorAccent2 = 6
msoThemeColorAccent3 = 7
msoThemeColorAccent4 = 8
msoThemeColorAccent5 = 9
msoThemeColorAccent6 = 10
msoThemeColorBackground1 = 14
msoThemeColorBackground2 = 16
msoThemeColorDark1 = 1
msoThemeColorDark2 = 3
msoThemeColorFollowedHyperlink = 12
msoThemeColorHyperlink = 11
msoThemeColorLight1 = 2
msoThemeColorLight2 = 4
msoThemeColorMixed = -2
msoThemeColorText1 = 13
msoThemeColorText2 = 15

# MsoThemeColorSchemeIndex enumeration
msoThemeAccent1 = 5
msoThemeAccent2 = 6
msoThemeAccent3 = 7
msoThemeAccent4 = 8
msoThemeAccent5 = 9
msoThemeAccent6 = 10
msoThemeDark1 = 1
msoThemeDark2 = 3
msoThemeFollowedHyperlink = 12
msoThemeHyperlink = 11
msoThemeLight1 = 2
msoThemeLight2 = 4

# MsoTriState enumeration
msoCTrue = 1
msoFalse = 0
msoTriStateMixed = -2
msoTriStateToggle = -3
msoTrue = -1

# MsoVerticalAnchor enumeration
msoAnchorBottom = 4
msoAnchorBottomBaseLine = 5
msoAnchorMiddle = 3
msoAnchorTop = 1
msoAnchorTopBaseline = 2
msoVerticalAnchorMixed = -2

# MsoWarpFormat enumeration
msoWarpFormat1 = 0
msoWarpFormat10 = 9
msoWarpFormat11 = 10
msoWarpFormat12 = 11
msoWarpFormat13 = 12
msoWarpFormat14 = 13
msoWarpFormat15 = 14
msoWarpFormat16 = 15
msoWarpFormat17 = 16
msoWarpFormat18 = 17
msoWarpFormat19 = 18
msoWarpFormat2 = 1
msoWarpFormat20 = 19
msoWarpFormat21 = 20
msoWarpFormat22 = 21
msoWarpFormat23 = 22
msoWarpFormat24 = 23
msoWarpFormat25 = 24
msoWarpFormat26 = 25
msoWarpFormat27 = 26
msoWarpFormat28 = 27
msoWarpFormat29 = 28
msoWarpFormat3 = 2
msoWarpFormat30 = 29
msoWarpFormat31 = 30
msoWarpFormat32 = 31
msoWarpFormat33 = 32
msoWarpFormat34 = 33
msoWarpFormat35 = 34
msoWarpFormat36 = 35
msoWarpFormat37 = 36
msoWarpFormat4 = 3
msoWarpFormat5 = 4
msoWarpFormat6 = 5
msoWarpFormat7 = 6
msoWarpFormat8 = 7
msoWarpFormat9 = 8
msoWarpFormatMixed = -2

# MsoWizardMsgType enumeration
msoWizardMsgLocalStateOff = 2
msoWizardMsgLocalStateOn = 1
msoWizardMsgResuming = 5
msoWizardMsgShowHelp = 3
msoWizardMsgSuspending = 4

# MsoZOrderCmd enumeration
msoBringForward = 2
msoBringInFrontOfText = 4
msoBringToFront = 0
msoSendBackward = 3
msoSendBehindText = 5
msoSendToBack = 1

# OutSpaceSlabStyle enumeration
OutSpaceSlabStyleError = 2
OutSpaceSlabStyleNormal = 0
OutSpaceSlabStyleWarning = 1

# RibbonControlSize enumeration
RibbonControlSizeLarge = 1
RibbonControlSizeRegular = 0

# SignatureDetail enumeration
sigdetApplicationName = 1
sigdetApplicationVersion = 2
sigdetColorDepth = 8
sigdetDelSuggSigner = 16
sigdetDelSuggSignerEmail = 20
sigdetDelSuggSignerEmailSet = 21
sigdetDelSuggSignerLine2 = 18
sigdetDelSuggSignerLine2Set = 19
sigdetDelSuggSignerSet = 17
sigdetDocPreviewImg = 10
sigdetHashAlgorithm = 14
sigdetHorizResolution = 6
sigdetIPCurrentView = 12
sigdetIPFormHash = 11
sigdetLocalSigningTime = 0
sigdetNumberOfMonitors = 5
sigdetOfficeVersion = 3
sigdetShouldShowViewWarning = 15
sigdetSignatureType = 13
sigdetSignedData = 9
sigdetVertResolution = 7
sigdetWindowsVersion = 4

# SignatureLineImage enumeration
siglnimgSignedInvalid = 3
siglnimgSignedValid = 2
siglnimgSoftwareRequired = 0
siglnimgUnsigned = 1

# SignatureProviderDetail enumeration
sigprovdetHashAlgorithm = 1
sigprovdetUIOnly = 2
sigprovdetUrl = 0

# SignatureType enumeration
sigtypeMax = 3
sigtypeNonVisible = 1
sigtypeSignatureLine = 2
sigtypeUnknown = 0

# XlDataLabelPosition enumeration
xlLabelPositionAbove = 0
xlLabelPositionBelow = 1
xlLabelPositionBestFit = 5
xlLabelPositionCenter = -4108
xlLabelPositionCustom = 7
xlLabelPositionInsideBase = 4
xlLabelPositionInsideEnd = 3
xlLabelPositionLeft = -4131
xlLabelPositionMixed = 6
xlLabelPositionOutsideEnd = 2
xlLabelPositionRight = -4152

# XlDisplayUnit enumeration
xlDisplayUnitCustom = -4114
xlDisplayUnitNone = -4142
xlHundredMillions = -8
xlHundreds = -2
xlHundredThousands = -5
xlMillionMillions = -10
xlMillions = -6
xlTenMillions = -7
xlTenThousands = -4
xlThousandMillions = -9
xlThousands = -3

# XlPivotFieldOrientation enumeration
xlColumnField = 2
xlDataField = 4
xlHidden = 0
xlPageField = 3
xlRowField = 1

# XlSizeRepresents enumeration
xlSizeIsArea = 1
xlSizeIsWidth = 2

# XlTimeUnit enumeration
xlDays = 0
xlMonths = 1
xlYears = 2



from matita.office import powerpoint as pp

def hello_world():
    pp_app = pp.Application()
    pp_app.visible = True
    prs = pp_app.presentations.add()
    sld = prs.slides.add(1, pp.ppLayoutText)
    shp = sld.shapes.addshape(pp.msoShapeRectangle, 100, 100, 200, 100)
    shp.text_frame.text_range.text = "Hello world!"

if __name__ == "__main__":
    hello_world()



import os

from matita.office import excel as xl

def generate_report():
    xlFormatCommas = 2
    file_path = os.path.dirname(os.path.abspath(__file__)) + "/tps00001__custom_19561911_linear_2_0.csv"

    xl_app = xl.Application()
    xl_app.visible = True
    
    data_wkb = xl_app.workbooks.Open(file_path, ReadOnly=True, Format=xlFormatCommas)
    data_wks = data_wkb.worksheets(1)
    data_tbl = data_wks.list_objects.add(
        SourceType=xl.xlSrcRange,
        Source=data_wks.usedrange,
        XlListObjectHasHeaders=xl.xlYes,
    )
    countries_clm = data_tbl.list_columns("Geopolitical entity (reporting)")
    population_clm = data_tbl.list_columns("OBS_VALUE")

    eu_countries = sorted(set(countries_clm.data_body_range.value))
    eu_countries = [x[0] for x in eu_countries.copy()]

    years = [x[0] for x in data_tbl.list_columns("TIME_PERIOD").data_body_range.value]
    start_year = int(min(years))
    end_year = int(max(years))
    num_rows = end_year - start_year + 1

    report_wkb = xl_app.workbooks.add()
    first_wks = report_wkb.worksheets(1)
    for country in eu_countries:
        last_wks = report_wkb.worksheets(report_wkb.worksheets.count)
        country_wks = report_wkb.worksheets.add(After=last_wks)

        # Add heading
        country_wks.name = country
        country_wks.cells(2, 2).value = f"Population of {country}"
        country_wks.rows.item(2).style = "Heading 1"
        country_wks.columns.item(1).column_width = 3

        # Add table
        country_wks.cells(4, 2).Value = "Year"
        country_wks.cells(4, 3).Value = "Population"
        country_wks.cells(5, 2).Value = start_year
        country_wks.cells(5, 2).data_series(Rowcol=xl.xlColumns, Type=xl.xlLinear, Date=xl.xlDay, Step=1, Stop=end_year)

        data_tbl.data_body_range.auto_filter(Field=countries_clm.Index, Criteria1=country)
        country_wks.cells(5, 3).resize(num_rows).value = population_clm.data_body_range.special_cells(xl.xlCellTypeVisible).value

        country_tbl = country_wks.list_objects.add(
            SourceType=xl.xlSrcRange,
            Source=country_wks.cells(4, 2).current_region,
            XlListObjectHasHeaders=xl.xlYes
        )
        country_tbl.list_columns("Population").data_body_range.number_format = "#,##0"

        shp = country_wks.shapes.add_chart2( 
            XlChartType=xl.xlLineMarkers,
            Left=country_wks.cells(4, 5).left,
            Top=country_wks.cells(4, 5).top,
        )
        c = shp.chart
        c.has_title = False
        chart_series = c.full_series_collection().item(1)
        chart_series.name = country_wks.cells(4, 2).address()
        chart_series.values  = f"'{country}'!{country_tbl.list_columns("Population").data_body_range.address()}"
        chart_series.x_values = f"'{country}'!{country_tbl.list_columns("Year").data_body_range.address()}"

    first_wks.delete()
    data_wkb.close(False)
    report_wkb.worksheets(1).activate()

if __name__ == "__main__":
    generate_report()



import logging
import os
import string

from .markdown import MarkdownTree

def page_filename_to_key(filename):
    key = filename.removesuffix(".md")
    # Remove portions of the filename which are not part of the api name
    # e.g. "(Object)" from "Excel.Application(Object)"
    key = key.split("(", 1)[0]
    return key

def camel_to_snake_case(s):
    """anyVariableName -> any_variable_name"""

    snake_s =  "".join(
        [f"_{c.lower()}" if c in string.ascii_uppercase else c for c in s]
    )
    snake_s = snake_s.strip("_")
    return snake_s

class DocPage:

    RESERVED_WORDS = ["and", "class", "from", "import", "property", "or"]

    def __init__(self, markdown_src):
        self.md = markdown_src
        self.md_tree = None
        self.title = None
        self.api_name = None

        self.module_name = None
        self.object_name = None
        self.property_name = None
        self.method_name = None
        
        self.is_object = None
        self.is_collection = None
        self.is_property = None
        self.is_method = None
        self.is_enumeration = None

        self.is_read_only_property = None
        self.has_return_value = None
        self.property_class = None
        self.return_value_class = None

        self.enumeration_values = None

        self.properties = []
        self.parameters = []
        self.methods = []
    
    def process_page(self):
        try:
            self.md_tree  = MarkdownTree(self.md)
        except Exception as e:
            logging.warning("Failed building MarkdownTree, raising an exception.")
            raise(e)
        
        self.title = self.md_tree .front_matter.variables["title"]
        if "api_name" in self.md_tree.front_matter.variables:
            self.api_name = self.md_tree.front_matter.variables["api_name"]
        self.process_title()
        self.process_api_name()

        if self.is_enumeration:
            self.process_enumeration()

        # Retrieve information from the opening paragraph
        # Examples
        # -------------------------------------------
        # A collection of all the **[Worksheet](Excel.Worksheet.md)** objects in the specified or active workbook. Each **Worksheet** object represents a worksheet.
        # Returns a collection of **[ListObject](Excel.ListObject.md)** objects on the worksheet. Read-only **[ListObjects](excel.listobjects.md)** collection.
        # Returns or sets a **String** value that represents the object name.
        # Returns a **[Range](Excel.Range(object).md)** object that represents all the cells on the worksheet (not just the cells that are currently in use).
        # Returns a **Range** object that represents the columns in the specified range.
        # -------------------------------------------
        p = self.md_tree.sections_by_level(1)[0].paragraphs[0].txt
        # Check whether the object is a collection
        self.is_collection = p.startswith("A collection")

        # Get the return type of the property (if any)
        if "Returns" in p:
            property_class = None
            if "**[" in p:
                property_class = p.split("**[", 1)[1].split("]", 1)[0]
            elif "**" in p:
                property_class = p.split("**", 1)[1].split("**", 1)[0]
            if "Returns a collection of" in p and property_class is not None:
                if not property_class.endswith("s"):
                    property_class += "s"
            if property_class is not None:
                if property_class.lower() in ["boolean", "variant", "string", "long", "double", "single", "integer"]:
                    property_class = None
            self.property_class = property_class

        # Check whether the property is read only
        if "Returns or sets" in p or "Read/write" in p:
            self.is_read_only_property = False
        else:
            self.is_read_only_property = True

        # Find the parameters of a property.
        # The "Syntax" section should look like this.
        # -------------------------------------------
        # ## Syntax
        # _expression_.**Range** (_Cell1_, _Cell2_)
        # -------------------------------------------
        sections = self.md_tree.sections_by_title("Syntax")
        if len(sections) > 0:
            line = sections[0].paragraphs[0].txt.replace("()", "")
            if "(" in line:
                parameters = line.split("(", 1)[1].split(")", 1)[0]
                parameters = parameters.split(", ")
                parameters = [p.strip(" _`[]*\\").replace("\\_", "_").strip("_ ") for p in parameters]
                for p in parameters:
                    if "," in p:
                        logging.warning(f"Parsing error in '{self.api_name}'. Parameter `{p}` ignored. This is probably due to a formatting error in the source file.")
                        parameters.remove(p)
                self.parameters = parameters
        
        # Find the return value of a property. The section looks like this:
        # Example
        # -------------------------------------------
        # ## Return value
        # A **[Workbook](Excel.Workbook.md)** object that represents the new workbook.
        # -------------------------------------------
        self.has_return_value = False
        sections = self.md_tree.sections_by_title("Return value")
        if len(sections) > 0:
            self.has_return_value = True
            s = sections[0]
            line = s.paragraphs[0].txt.splitlines()[0]
            if any(word in line.lower() for word in ["nothing", "none", "false"]):
                self.return_value_class = None
            elif "**[" in line:
                self.return_value_class = line.split("**[", 1)[1].split("](")[0]
            elif "**" in line:
                self.return_value_class = line.split("**", 1)[1].split("**", 1)[0]
            elif " " in line.strip():
                logging.warning(f"Unexpected format in 'Return value' section, could not parse '{self.title}': '{line}'")
            else:
                self.return_value_class = line
            if self.return_value_class is not None:
                if self.return_value_class.lower() in ["boolean", "variant", "string", "long", "double", "single", "integer"]:
                    self.return_value_class = None

    def process_title(self):
        if self.title is None:
            self.is_object = None
            self.is_property = None
            self.is_method = None
            self.is_enumeration = None
            return
        else:
            self.is_object = False
            self.is_property = False
            self.is_method = False
            self.is_enumeration = False

        if "object" in self.title:
            self.is_object = True
        elif "property" in self.title:
            self.is_property = True
        elif "method" in self.title:
            self.is_method = True
        elif "enumeration" in self.title:
            self.is_enumeration = True

        try:
            title_split = self.title.split(" ")[0].split(".")
            self.object_name = title_split[0]
            if len(title_split) > 1:
                if title_split[1].lower() != "enumerations":
                    if self.is_property:
                        self.property_name = title_split[1]
                    elif self.is_method:
                        self.method_name = title_split[1]
        except Exception as e:
            logging.error(f"Failed parsing title {self.title}. {e}")

    def process_api_name(self):
        if self.api_name is None:
            return
        api_name_split = self.api_name.split(".")
        self.module_name = api_name_split[0]
    
    def process_enumeration(self):
        """Parse enumeration information. Assumes that the page refers to an enumeration.

        Example: [XlReferenceType enumeration (Excel)](https://learn.microsoft.com/en-gb/office/vba/api/excel.xlreferencetype)
        """

        # Find table paragraph in the first section
        table = None
        for p in self.md_tree.sections_by_level(1)[0].paragraphs:
            if p.is_table:
                try:
                    table = p.table
                except Exception as e:
                    logging.WARNING(f"Failed parsing values table for enumeration {self.api_name}. {e}")
                break
        
        if "Name" not in table.rows[0] or "Value" not in table.rows[0]:
            logging.WARNING(f"Unexpected header for values table for enumeration {self.api_name}.")
            return
        
        self.enumeration_values = {}
        for r in table.rows[1:]:
            key = r[0].strip("*")
            value = r[1]
            if value.lstrip("-").isnumeric():
                if int(value) == float(value):
                    value = int(value)
                else:
                    value = float(value)
            self.enumeration_values[key] = value
    
    def parent_object_key(self):
        if (self.module_name and self.object_name) is None:
            return None
        return ".".join([self.module_name, self.object_name]).lower()
    
    def to_dict(self):
        return {
            "title": self.title,
            "module_name": self.module_name,
            "object_name": self.object_name,
            "property_name": self.property_name,
            "method_name": self.method_name,
            "is_object": self.is_object,
            "is_collection": self.is_collection,
            "is_method": self.is_method,
            "is_property": self.is_property,
            "is_enumeration": self.is_enumeration,
            "api_name": self.api_name,
            "is_collection": self.is_collection,
            "is_read_only_property": self.is_read_only_property,
            "parent object key": self.parent_object_key(),
            "property_class": self.property_class,
            "return_value_class": self.return_value_class,
            "enumeration_values": self.enumeration_values,
            "properties": [page.title for page in self.properties],
            "parameters": self.parameters,
            "methods": [page.title for page in self.methods],
        }
    
    def to_python(self):
        """Return source code of a python class based on the page"""

        if self.is_enumeration:
            code = []
            code.append(f"# {self.object_name} enumeration")
            for key, value in self.enumeration_values.items():
                if type(value) == str:
                    value = f"\"{value}\""
                code.append(f"{key} = {value}")
            code.append("")
            return "\n".join(code)

        if not self.is_object:
            return None

        code = []
        if self.object_name == "Application":
            code.append(f"class {self.object_name}:")
            code.append(f"")
            code.append(f"    def __init__(self, application=None):")
            code.append(f"        if application is None:")
            code.append(f"            self.com_object = win32com.client.gencache.EnsureDispatch(\"{self.module_name}.Application\")")
            code.append(f"        else:")
            code.append(f"            self.com_object = application")
            code.append(f"")
        else:
            code.append(f"class {self.object_name}:")
            code.append(f"")
            code.append(f"    def __init__(self, {self.object_name.lower()}=None):")
            code.append(f"        self.com_object= {self.object_name.lower()}")
            code.append(f"")

        # Call method for collections
        if self.is_collection:
            item_class = self.object_name
            if self.object_name.endswith("s"):
                item_class = self.object_name[:-1]
            elif self.object_name.endswith("Collection"):
                item_class = self.object_name.removesuffix("Collection")
            else:
                logging.warning(f"Unexpected collection name, unable to identify item class of '{self.process_api_name}'.")
            code.append(f"    def __call__(self, item):")
            code.append(f"        return {item_class}(self.com_object(item))")
            code.append(f"")

        code += self.to_python_properties()
        code += self.to_python_methods()
        code.append("")
        return "\n".join(code)

    def parameters_code(self):
        """Return code to include parameters as arguments of a property or method"""
        return "=None, ".join(self.parameters) + "=None"

    def to_python_arguments_expansion(self):
        """Return python code to expand arguments for COM calls"""
        return f"        arguments = com_arguments([unwrap(a) for a in [{", ".join(self.parameters)}]])"

    def to_python_property_getter(self):
        """Return python code for getter and setter of the property"""
        if not self.is_property:
            logging.warning(f"Property '{self.title}' ignored when exporting getter for '{self.object_name}', because it is not a property.")
            return []
        
        if self.property_name in self.RESERVED_WORDS:
            logging.warning(f"Property '{self.title}' ignored when exporting getter for '{self.object_name}', because it is a reserved word in Python.")
            return []
        
        code = []
        # Getter method - no arguments
        if len(self.parameters) == 0 or not self.is_read_only_property:
            if self.property_class is not None:
                code.append("    @property")
                code.append(f"    def {self.property_name}(self):")
                code.append(f"        return {self.property_class}(self.com_object.{self.property_name})")
            else:
                code.append("    @property")
                code.append(f"    def {self.property_name}(self):")
                code.append(f"        return self.com_object.{self.property_name}")
            code.append(f"")
        # Getter method - with arguments
        else:
            code.append(f"    def {self.property_name}(self, {self.parameters_code()}):")
            code.append(self.to_python_arguments_expansion())
            if self.property_class is not None:
                code.append(f"        if hasattr(self.com_object, \"Get{self.property_name}\"):")
                code.append(f"            return {self.property_class}(self.com_object.Get{self.property_name}(*arguments))")
                code.append(f"        else:")
                code.append(f"            return {self.property_class}(self.com_object.{self.property_name}(*arguments))")
            else:
                code.append(f"        if hasattr(self.com_object, \"Get{self.property_name}\"):")
                code.append(f"            return self.com_object.Get{self.property_name}(*arguments)")
                code.append(f"        else:")
                code.append(f"            return self.com_object.{self.property_name}(*arguments)")
            code.append(f"")

        return code
        
    def to_python_property_setter(self):
        """Return python code for setter of the property"""
        if not self.is_property:
            logging.info(f"Property '{self.title}' ignored when exporting setter for '{self.object_name}', because it is not a property.")
            return []
        
        # If the property is editable, it must have a setter method.
        code = []
        if not self.is_read_only_property:
            code.append(f"    @{self.property_name}.setter")
            code.append(f"    def {self.property_name}(self, value):")
            code.append(f"        self.com_object.{self.property_name} = value")
            code.append("")

        return code

    def to_python_property_aliases(self):
        """Return python code of lower case aliases if the DocPage is a property"""

        if not self.is_property:
            logging.info(f"Property '{self.title}' ignored when exporting aliases for '{self.api_name}', because it is not a property.")
            return []

        # Don't create lower case alias for reserved words in Python
        # E.g. `Access.BoundObjectFrame.Class`
        if self.property_name.lower() in self.RESERVED_WORDS:
            logging.info(f"Aliases for property '{self.property_name}' of '{self.api_name}' ignored, because it is a reserved word in Python.")
            return []
        
        aliases = []
        if self.property_name != self.property_name.lower():
            aliases.append(self.property_name.lower())
        if self.property_name.lower() != camel_to_snake_case(self.property_name):
            aliases.append(camel_to_snake_case(self.property_name))

        code = []
        for alias in aliases:
            # Getter method
            if len(self.parameters) == 0 or not self.is_read_only_property:
                code.append(f"    @property")
                code.append(f"    def {alias}(self):")
                code.append(f"        \"\"\"Alias for {self.property_name}\"\"\"")
                code.append(f"        return self.{self.property_name}")
            else:
                code.append(f"    def {alias}(self, {self.parameters_code()}):")
                code.append(f"        \"\"\"Alias for {self.property_name}\"\"\"")
                code.append(f"        arguments = [{", ".join(self.parameters)}]")
                code.append(f"        return self.{self.property_name}(*arguments)")
            code.append(f"")

            # Setter method
            if not self.is_read_only_property:
                code.append(f"    @{alias}.setter")
                code.append(f"    def {alias}(self, value):")
                code.append(f"        \"\"\"Alias for {self.property_name}.setter\"\"\"")
                code.append(f"        self.{self.property_name} = value")
                code.append(f"")

        return code

    def to_python_properties(self):
        """Return python code for all properties of the object"""
        code = []
        for p in self.properties:
            if p.property_name is None:
                logging.info("Property '{}' ignored when exporting '{}', because property_name is None.".format(p.title, self.title))
                continue

            code += p.to_python_property_getter()
            code += p.to_python_property_setter()
            code += p.to_python_property_aliases()

        return code
    
    def to_python_method_function(self, parent_is_collection=False):
        """Return python code for a single method of the object"""
        
        if not self.is_method:
            logging.info(f"Method '{self.title}' ignored when exporting for '{self.object_name}', because it is not a method.")
            return []
        
        code = []
        if len(self.parameters) == 0:
            code.append(f"    def {self.method_name}(self):")
            code_line = f"self.com_object.{self.method_name}()"
        else:
            code.append(f"    def {self.method_name}(self, {self.parameters_code()}):")
            code.append(self.to_python_arguments_expansion())
            code_line = f"self.com_object.{self.method_name}(*arguments)"
        if self.has_return_value:
            if self.return_value_class is not None:
                code_line = f"{self.return_value_class}({code_line})"
            # Certain methods of a collection can only return certain types
            # e.g. `Worksheets.Add`` returns a `Worksheet`
            # `.startswith("Open") is for methods like `Workbooks.OpenText`
            elif parent_is_collection and (self.method_name == "Add" or self.method_name.startswith("Open")):
                code_line = f"{self.object_name[:-1]}({code_line})"
            code_line = "return " + code_line
        code_line = " "*8 + code_line
        code.append(code_line)
        code.append("")
        return code

    def to_python_method_function_aliases(self):

        code = []
        if self.method_name.lower() in self.RESERVED_WORDS:
            logging.info(f"Aliases for method '{self.method_name}' of '{self.object_name}' ignored, because it is a reserved word in Python.")
            return code
        
        aliases = []
        if self.method_name != self.method_name.lower():
            aliases.append(self.method_name.lower())
        if self.method_name.lower() != camel_to_snake_case(self.method_name):
            aliases.append(camel_to_snake_case(self.method_name))

        for alias in aliases:
            if len(self.parameters) == 0:
                code.append(f"    def {alias}(self):")
                code.append(f"        \"\"\"Alias for {self.method_name}\"\"\"")
                code.append(f"        return self.{self.method_name}()")
            else:
                code.append(f"    def {alias}(self, {self.parameters_code()}):")
                code.append(f"        \"\"\"Alias for {self.method_name}\"\"\"")
                code.append(f"        arguments = [{", ".join(self.parameters)}]")
                code.append(f"        return self.{self.method_name}(*arguments)")
            code.append("")

        return code
    
    def to_python_methods(self):
        """Return python code for all methods of the object"""
        code = []
        for m in self.methods:
            code += m.to_python_method_function(parent_is_collection=self.is_collection)
            code += m.to_python_method_function_aliases()
        return code

class VbaDocs:

    def __init__(self):
        self.pages = dict()

    def read_directory(self, path):
        with os.scandir(path) as it:
            for entry in it:
                if entry.is_file():
                    print(f"Parsing {entry.name}...")
                    page_key = page_filename_to_key(entry.name).lower()
                    if page_key in ["word.break", "word.global"]:
                        logging.info("Ignoring page '{}', because it conflicts with a Python keyword.".format(entry.name))
                    elif page_key in ["access.report.circle", "access.report.line"]:
                        logging.info("Ignoring page '{}', because non-scalar arguments are not implemented.".format(entry.name))
                    elif "-" in page_key:
                        logging.info("Ignoring page '{}', because the object name includes a dash.".format(entry.name))
                    else:
                        self.pages[page_key] = DocPage(open(entry, "rt", encoding="utf8").read())
    
    def process_pages(self):
        pages_to_remove = []
        for page_key, page in self.pages.items():
            try:
                page.process_page()
                # Remove pages without `api_name`
                if page.api_name is None:
                    logging.warning(f"Attribute `api_name` not found for {page_key}, ignoring.")
                    pages_to_remove.append(page_key)
            except Exception as e:
                logging.error(f"Failed processing page: '{page_key}'. {e}")
        for key in pages_to_remove:
            del self.pages[key]
        # Populate properties and methods of objects
        for page_key, page in self.pages.items():
            parent_object_key = page.parent_object_key()
            if parent_object_key is not None:
                if page.is_property:
                    if parent_object_key in self.pages:
                        self.pages[parent_object_key].properties.append(page)
                    else:
                        logging.warning(f"Parent object '{parent_object_key}' for property '{page_key}' not found.")
                elif page.is_method:
                    if parent_object_key in self.pages:
                        self.pages[parent_object_key].methods.append(page)
                    else:
                        logging.warning(f"Parent object '{parent_object_key}' for method '{page_key}' not found.")
            else:
                if page.is_property or page.is_method:
                    logging.warning(f"Page'{page_key}' is a property or method, but the key of the parent object of is None.")

        # Remove invalid class types
        for page in self.pages.values():
            if page.property_class is not None:
                if f"{page.module_name}.{page.property_class}".lower() not in self.pages:
                    logging.warning(f"Removing 'property_class' from {page.api_name}. Class '{page.property_class}' not found.")
                    page.property_class = None
            if page.return_value_class is not None:
                if f"{page.module_name}.{page.return_value_class}".lower() not in self.pages:
                    logging.warning(f"Removing 'return_value_class' from {page.api_name}. Class '{page.return_value_class}' not found.")
                    page.return_value_class = None
        self.apply_manual_adjustments()
    
    def to_dict(self):
        dictionaries = [page.to_dict() for page in self.pages.values()]
        keys_and_values = zip(self.pages.keys(), dictionaries)
        return {key: value for key, value in keys_and_values}

    def apply_manual_adjustments(self):
        # Add parameters for Cell properties, whose parameters are not properly imported
        for p in self.pages.values():
            if p.property_name == "Cells" and len(p.parameters) == 0:
                p.parameters = ["RowIndex", "ColumnIndex"]

    def to_python(self, application):
        code = [
            "from . import com_arguments, unwrap",
            "from .office import *",
            "",
            "import win32com.client",
            "",]
        for page_key, page in self.pages.items():
            if page.module_name is None:
                continue
            if page.module_name.lower() != application.lower():
                continue
            # For the `office` module, only include enumerations
            if page.module_name.lower() == "office" and not page.is_enumeration:
                continue
            try:
                page_code = page.to_python()
            except Exception as e:
                logging.warning(f"Can't export '{page_key}' to python code. {e}")
            else:
                if page_code is not None:
                    code.append(page_code)
        return "\n".join(code)
    
    def apply_manual_adjustments(self):
        # Add parameters for Cell properties, whose parameters are not properly imported
        for p in self.pages.values():
            if p.property_name == "Cells" and len(p.parameters) == 0:
                p.parameters = ["RowIndex", "ColumnIndex"]



import json
import logging
import os
import sys

sys.path.insert(0, os.path.join(os.path.dirname(__file__), "..", "src"))

from matita.reference.models import VbaDocs

def main():
    os.makedirs("logs", exist_ok=True)
    logging.basicConfig(
        level = logging.INFO,
        format = "%(asctime)s [%(levelname)s] %(message)s",
        handlers = [logging.FileHandler("logs/matita.log", mode="w")],
    )

    print("Parsing Office VBA Reference...")
    docs = VbaDocs()
    docs.read_directory("office-vba-reference/api")
    print("Analysing Office VBA Reference...")
    docs.process_pages()
    os.makedirs("data", exist_ok=True)
    json.dump(docs.to_dict(), open("data/office-vba-api.json", "wt"), indent=4)
    for app in ["Excel", "Word", "PowerPoint", "Outlook", "Access", "Office"]:
        print(f"Generating matita.office.{app.lower()} module...")
        with open(f"src/matita/office/{app.lower()}.py", "wt") as f:
            f.write(docs.to_python(application=app))

if __name__ == "__main__":
    main()



from matita.office import outlook as ol

def hello_world():
    ol_app = ol.Application()
    mail = ol.MailItem(ol_app.create_item(ol.olMailItem))
    mail.body = "Hello world!"
    mail.display()

if __name__ == "__main__":
    hello_world()



import unittest

from matita.reference import MarkdownTree

test_text = open("tests/test_doc_page.md", "rt").read()
test_tree = MarkdownTree(test_text)

class TestParser(unittest.TestCase):

    def test_front_matter(self):
        fm = test_tree.front_matter.variables
        self.assertEqual(fm["title"], "Lorem Ipsum")
        self.assertEqual(fm["tags"], "consectetaur adipisicing")
        self.assertEqual(fm["color"], "sed do eiusmod tempor")
    
    def test_sections(self):
        self.assertEqual(test_tree.sections[0].title, "")
        self.assertEqual(test_tree.sections[0].paragraphs[0].txt, "At vero eos et accusamus.")
        self.assertEqual(test_tree.sections[0].paragraphs[1].txt, "Praesentium voluptatum.")
        self.assertEqual(test_tree.sections[1].title, "Section A")
        self.assertEqual(test_tree.sections[1].paragraphs[0].txt, "Lorem ipsum\ndolor sit amet.")
        self.assertEqual(test_tree.sections[1].level, 1)
        self.assertEqual(test_tree.sections[2].title, "Section B")
        self.assertEqual(test_tree.sections[2].paragraphs[0].txt, "Excepteur sint occaecat.")
        self.assertEqual(test_tree.sections[2].level, 2)
        self.assertEqual(test_tree.sections[6].title, "Section F")
        self.assertEqual(test_tree.sections[6].paragraphs[0].txt, "Sed quia consequuntur.")
        self.assertEqual(test_tree.sections[6].level, 2)
    
    def test_sections_by_level(self):
        self.assertEqual(test_tree.sections_by_level(2)[1].title, "Section C")
    
    def test_sections_by_title(self):
        self.assertEqual(test_tree.sections_by_title("on C")[0].title, "Section C")

    def test_table_parsing(self):
        table_paragraph = test_tree.sections_by_title("Section G")[0].paragraphs[1]
        self.assertTrue(table_paragraph.is_table)
        rows = table_paragraph.table.rows
        self.assertEqual(rows[0][0], "Lorem")
        self.assertEqual(rows[2][1], "e")
        self.assertEqual(rows[3][2], "i")



import inspect
import unittest

from matita.office import access as ac, excel as xl, outlook as ol, powerpoint as pp, word as wd


class TestAccess(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        cls.ac_app = ac.Application()
        cls.ac_app.Visible = True

    @classmethod
    def tearDownClass(cls):
        cls.ac_app.Quit()
    
    def test_aliases(self):
        membs = inspect.getmembers(ac.Application)
        membs = [m[0] for m in membs.copy()]
        # Parameter
        self.assertIn("AppIcon", membs)
        self.assertIn("appicon", membs)
        self.assertIn("app_icon", membs)
        #Method
        self.assertIn("FileDialog", membs)
        self.assertIn("filedialog", membs)
        self.assertIn("file_dialog", membs)

    def test_access_visibility(self):
        self.assertTrue(self.ac_app.Visible)

class TestExcel(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.xl_app = xl.Application()
        cls.xl_app.visible = True

    @classmethod
    def tearDownClass(cls):
        cls.xl_app.Quit()

    def test_hello_world(self):
        wkb = self.xl_app.workbooks.Add()
        wks = wkb.worksheets(1)
        c = wks.cells(1,1)
        c.value = "Hello world!"
        self.assertEqual(c.com_object.Value, "Hello world!")
        wkb.Close(SaveChanges=False)

    def test_excel_types(self):
        wkb = self.xl_app.workbooks.add()
        wks = wkb.worksheets(1)

        cell_str = wks.range("A1")
        cell_int = wks.range("A2")
        cell_float = wks.range("A3")
        cell_bool = wks.range("A4")

        cell_str.value = "ciao"
        cell_int.value = 123
        cell_float.value = 3.14159
        cell_bool.value = True

        self.assertEqual(cell_str.value, "ciao")
        self.assertEqual(cell_int.value, 123)
        self.assertAlmostEqual(cell_float.value, 3.14159)
        self.assertEqual(cell_bool.value, True)
        self.assertIs(type(wks.list_objects), xl.ListObjects)

        wkb.Close(SaveChanges=False)

    def test_range_address(self):
        wkb = self.xl_app.Workbooks.Add()
        wks = wkb.Worksheets(1)

        r = wks.range("B2:D4")
        self.assertEqual(r.Address(), "$B$2:$D$4")
        self.assertEqual(r.address(), "$B$2:$D$4")
        self.assertEqual(r.Address(ReferenceStyle=xl.xlR1C1), "R2C2:R4C4")
        self.assertEqual(r.address(ReferenceStyle=xl.xlR1C1), "R2C2:R4C4")

        wkb.Close(SaveChanges=False)

    def test_excel_aliases(self):
        wkb = self.xl_app.Workbooks.Add()

        self.assertIs(type(wkb.Worksheets.Add()), xl.Worksheet)
        self.assertIs(type(wkb.Worksheets.add()), xl.Worksheet)

        wkb.Close(SaveChanges=False)

    def test_excel_constants(self):
        self.assertEqual(xl.xlAscending, 1)
        self.assertEqual(xl.xlDescending, 2)

    def test_excel_com_object(self):
        wkb = self.xl_app.Workbooks.Add()
        self.assertIs(type(wkb), xl.Workbook)
        self.assertIn("win32", str(type(wkb.com_object)))
        wkb.Close(SaveChanges=False)

    def test_range_operations(self):
        wkb = self.xl_app.Workbooks.Add()
        wks = wkb.worksheets(1)
        rng = wks.cells(2,3)
        self.assertEqual(rng.resize(4,5).address(), "$C$2:$G$5")

        rng = wks.range("$A$1:$C$2")
        self.assertEqual(rng.offset(2,3).address(), "$D$3:$F$4")

        cell1 = wks.cells(2,2)
        cell2 = wks.cells(5,5)
        rng = wks.range(cell1, cell2)
        self.assertEqual(rng.address(), "$B$2:$E$5")

        rng1 = wks.range("B2:D5")
        rng2 = wks.range("C4:E7")
        rng = self.xl_app.intersect(rng1, rng2)
        self.assertEqual(rng.address(), "$C$4:$D$5")

        cell1 = wks.cells(1,1)
        cell2 = wks.cells(2,2)
        rng = self.xl_app.intersect(cell1, cell2)
        self.assertIsNone(rng.com_object)

class TestOutlook(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ol_app = ol.Application()

    @classmethod
    def tearDownClass(cls):
        cls.ol_app.Quit()
        
    def test_hello_world(self):
        mail = ol.MailItem(self.ol_app.create_item(ol.olMailItem))
        mail.body = "Hello world!"
        self.assertTrue(mail.com_object.Body.startswith("Hello world!"))

class TestPowerPoint(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.pp_app = pp.Application()
        cls.pp_app.visible = True

    @classmethod
    def tearDownClass(cls):
        cls.pp_app.Quit()

    def test_hello_world(self):
        prs = self.pp_app.presentations.add()
        sld = prs.slides.add(1, pp.ppLayoutBlank)
        shp = sld.shapes.add_shape(pp.msoShapeRectangle, 30, 30 , 30, 30)
        shp.text_frame.text_range.text = "Hello world!"
        self.assertEqual(shp.text_frame.text_range.com_object.Text, "Hello world!")
        prs.close()

    def test_powerpoint(self):
        self.assertTrue(self.pp_app.visible)
        prs = self.pp_app.presentations.add()
        sld = prs.slides.add(1, pp.ppLayoutBlank)
        shp = sld.shapes.add_shape(pp.msoShapeRectangle, 30, 30 , 30, 30)
        eff = sld.timeline.main_sequence.add_effect(
            Shape=shp,
            effectId=pp.msoAnimEffectFly,
            Level=pp.msoAnimateLevelNone,
            trigger=pp.msoAnimTriggerAfterPrevious,
        )
        prs.close()
        
    def test_powerpoint_com_object(self):
        prs = self.pp_app.presentations.add()
        self.assertIs(type(prs), pp.Presentation)
        self.assertIn("win32", str(type(prs.com_object)))
        prs.close()

class TestWord(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        cls.wd_app = wd.Application()
        cls.wd_app.visible = True

    @classmethod
    def tearDownClass(cls):
        cls.wd_app.Quit(SaveChanges=wd.wdDoNotSaveChanges)

    def test_hello_world(self):
        doc = self.wd_app.documents.add()
        par = doc.content.paragraphs.add()
        par.range.text = "Hello world!"
        self.assertTrue(par.range.com_object.Text.startswith("Hello world!"))



from matita.office import excel as xl

def hello_world():
    xl_app = xl.Application()
    xl_app.visible = True

    wkb = xl_app.workbooks.Add()
    wks = wkb.worksheets(1)
    c = wks.cells(1,1)
    c.value = "Hello world!"

if __name__ == "__main__":
    hello_world()
